"""
The following contains a database of small molecules

Data for the G2 database are from
Raghavachari, Redfern, and Pople, J. Chem. Phys. Vol. 106, 1063 (1997).
See http://www.cse.anl.gov/Catalysis_and_Energy_Conversion/Computational_Thermochemistry.shtml for the original files.

All numbers are experimental values, except for coordinates, which are
MP2(full)/6-31G(d) optimized geometries

Atomic species:
ref: Curtiss et al. JCP 106, 1063 (1997).
'Enthalpy' is the experimental enthalpies of formation at 0K
'thermal correction' is the thermal corrections H(298)-H(0)

Molecular species:
ref: Staroverov et al. JCP 119, 12129 (2003)
'Enthalpy' is the experimental enthalpies of formation at 298K
'ZPE' is the zero-point energies
'thermal correction' is the thermal enthalpy corrections H(298K) - H_exp(0K)
ZPE and thermal corrections are estimated from B3LYP geometries and vibrations.

Data for extra systems are from CCCBDB: http://srdata.nist.gov/cccbdb/
Experimental ionization potentials are also from CCCBDB.
"""

from ase.atoms import Atoms, string2symbols

atoms = ['H','Li','Be','B','C','N','O','F','Na','Mg','Al','Si','P','S','Cl']
extra = ['Be2','C7NH5','BDA','biphenyl','C60']
g1 = ['LiH','BeH','CH','CH2_s3B1d','CH2_s1A1d','CH3','CH4','NH','NH2','NH3','OH','H2O','HF','SiH2_s1A1d','SiH2_s3B1d','SiH3','SiH4','PH2','PH3','SH2','HCl','Li2','LiF','C2H2','C2H4','C2H6','CN','HCN','CO','HCO','H2CO','CH3OH','N2','N2H4','NO','O2','H2O2','F2','CO2','Na2','Si2','P2','S2','Cl2','NaCl','SiO','CS','SO','ClO','ClF','Si2H6','CH3Cl','CH3SH','HOCl','SO2']
g2 = g1 + ['BF3','BCl3','AlF3','AlCl3','CF4','CCl4','OCS','CS2','COF2','SiF4','SiCl4','N2O','ClNO','NF3','PF3','O3','F2O','ClF3','C2F4','C2Cl4','CF3CN','C3H4_C3v','C3H4_D2d','C3H4_C2v','C3H6_Cs','C3H6_D3h','C3H8','butadiene','2-butyne','methylenecyclopropane','bicyclobutane','cyclobutene','cyclobutane','isobutene','trans-butane','isobutane','C5H8','C6H6','H2CF2','HCF3','H2CCl2','HCCl3','H3CNH2','CH3CN','CH3NO2','CH3ONO','CH3SiH3','HCOOH','HCOOCH3','CH3CONH2','CH2NHCH2','NCCN','C2H6NH','CH3CH2NH2','H2CCO','CH2OCH2','CH3CHO','OCHCHO','CH3CH2OH','CH3OCH3','CH2SCH2','C2H6SO','CH3CH2SH','CH3SCH3','H2CCHF','CH3CH2Cl','H2CCHCl','H2CCHCN','CH3COCH3','CH3COOH','CH3COF','CH3COCl','C3H7Cl','C2H6CHOH','CH3CH2OCH3','C3H9N','C4H4O','C4H4S','C4H4NH','C5H5N','H2','SH','CCH','C2H3','CH3CO','H2COH','CH3O','CH3CH2O','CH3S','C2H5','C3H7','C3H9C','NO2']
g3 = g2 + ['Butadiene_1_2','Isoprene','Cyclopentane','n_Pentane','Neopentane','Cyclohexadiene_1_3','Cyclohexadiene_1_4','Cyclohexane','n_Hexane','Methyl_pentane_3','Toluene','n_Heptane','Cyclooctatetraene','n_Octane','Naphthalene','Azulene','Methyl_acetate','t_Butanol','Aniline','Phenol','Divinyl_ether','Tetrahydrofuran','Cyclopentanone','Benzoquinone_1_4','Pyrimidine','Dimethyl_sulfone','Chlorobenzene','Succinonitrile','Pyrazine','Acetyl_acetylene','Crotonaldehyde','Acetic_anhydride','Dihydrothiophene_2_5','Methyl_propanenitrile_2','Methyl_ethyl_ketone','Isobutyraldehyde','dioxane_1_4','Tetrahydrothiophene','t_Butyl_chloride','n_Butyl_chloride','Tetrahydropyrrole','Nitrobutane_2','Diethyl_ether','Dimethoxy_ethane_1_1','t_Butanethiol','Diethyl_disulfide','t_Butylamine','Tetramethylsilane','Methyl_thiophene','N_methyl_pyrrole','Tetrahydropyran','Diethyl_ketone','Isopropyl_acetate','Tetrahydrothiopyran','Piperidine','t_Butyl_methyl_ether','Difluorobenzene_1_3','Difluorobenzene_1_4','Fluorobenzene','Diisopropyl_ether','PF5','SF6','P4_Td','SO3_D3h','SCl2','POCl3','PCl5','SO2Cl2','PCl3','S2Cl2','SiCl2_1A1','CF3Cl','C2F6','CF3','Phenyl']

data = {
'H': {
    'name': 'Hydrogen',
    'magmom': 1,
    'enthalpy': 51.63,
    'thermal correction': 1.01,
    'ionization energy': 13.60},
'Li': {
    'name': 'Lithium',
    'magmom': 1,
    'enthalpy': 37.69,
    'thermal correction': 1.10,
    'ionization energy': 5.39},
'Be': {
    'name': 'Beryllium',
    'magmom': 0,
    'enthalpy': 76.48,
    'thermal correction': 0.46,
    'ionization energy': 9.32},
'B': {
    'name': 'Boron',
    'magmom': 1,
    'enthalpy': 136.20,
    'thermal correction': 0.29,
    'ionization energy': 8.30},
'C': {
    'name': 'Carbon',
    'magmom': 2,
    'enthalpy': 169.98,
    'thermal correction': 0.25,
    'ionization energy': 11.26},
'N': {
    'name': 'Nitrogen',
    'magmom': 3,
    'enthalpy': 112.53,
    'thermal correction': 1.04,
    'ionization energy': 14.53},
'O': {
    'name': 'Oxygen',
    'magmom': 2,
    'enthalpy': 58.99,
    'thermal correction': 1.04,
    'ionization energy': 13.62},
'F': {
    'name': 'Fluorine',
    'magmom': 1,
    'enthalpy': 18.47,
    'thermal correction': 1.05,
    'ionization energy': 17.42},
'Na': {
    'name': 'Sodium',
    'magmom': 1,
    'enthalpy': 25.69,
    'thermal correction': 1.54,
    'ionization energy': 5.14},
'Mg': {
    'name': 'Magnesium',
    'magmom': 0,
    'enthalpy': 34.87,
    'thermal correction': 1.19,
    'ionization energy': 7.65},
'Al': {
    'name': 'Aluminium',
    'magmom': 1,
    'enthalpy': 78.23,
    'thermal correction': 1.08,
    'ionization energy': 5.99},
'Si': {
    'name': 'Silicon',
    'magmom': 2,
    'enthalpy': 106.60,
    'thermal correction': 0.76,
    'ionization energy': 8.15},
'P': {
    'name': 'Phosphorus',
    'magmom': 3,
    'enthalpy': 75.42,
    'thermal correction': 1.28,
    'ionization energy': 10.49},
'S': {
    'name': 'Sulfur',
    'magmom': 2,
    'enthalpy': 65.66,
    'thermal correction': 1.05,
    'ionization energy': 10.36},
'Cl': {
    'name': 'Chlorine',
    'magmom': 1,
    'enthalpy': 28.59,
    'thermal correction': 1.10,
    'ionization energy': 12.97},
'Be2': {
    'description': "Diatomic Beryllium",
    'name': "Be_2",
    'enthalpy': 155.1,
    'ZPE': 1.0000,
    'thermal correction': 5.0600,
    'symbols': 'BeBe',
    'magmoms': None,
    'positions': [[ 0.  ,  0.  ,  1.0106],
                  [ 0.  ,  0.  , -1.0106]]},
'C7NH5': {
    'description': "Benzonitride",
    'name': "C_7NH_5",
    'symbols': 'C7NH5',
    'magmoms': None,
    'positions': [[ -1.593581, -1.142601, 0.],
                  [ -2.235542,  0.095555, 0.],
                  [ -0.204885, -1.210726, 0.],
                  [  0.549645, -0.025355, 0.],
                  [  1.976332, -0.085321, 0.],
                  [ -0.099258,  1.220706, 0.],
                  [ -1.488628,  1.273345, 0.],
                  [  3.136871, -0.128138, 0.],
                  [ -2.177996, -2.060896, 0.],
                  [ -3.323594,  0.141242, 0.],
                  [  0.301694, -2.173705, 0.],
                  [  0.488716,  2.136782, 0.],
                  [ -1.987765,  2.240495, 0.]]},
'BDA': {
    'description': "1,4-Benzodiamine",
    # aka p-Aminoaniline; p-Benzenediamine; p-Diaminobenzene;
    #     p-Phenylenediamine; Paraphenylen-diamine
    'name': "BDA",
    # PBE-gpaw relaxed
    'symbols': 'C6H4N2H4',
    'magmoms': None,
    'positions': [[ 0.004212,  1.406347,  0.061073],
                  [ 1.193490,  0.687096,  0.029481],
                  [ 1.190824, -0.690400, -0.028344],
                  [ 0.000295, -1.406191, -0.059503],
                  [-1.186974, -0.685668, -0.045413],
                  [-1.185376,  0.690203,  0.009452],
                  [ 2.147124,  1.219997,  0.064477],
                  [ 2.141593, -1.227477, -0.054266],
                  [-2.138408, -1.222814, -0.095050],
                  [-2.137740,  1.226930,  0.023036],
                  [-0.006314,  2.776024,  0.186278],
                  [-0.007340, -2.777839, -0.159936],
                  [ 0.844710, -3.256543,  0.110098],
                  [-0.854965, -3.253324,  0.130125],
                  [ 0.845826,  3.267270, -0.055549],
                  [-0.854666,  3.254654, -0.092676]]},
'biphenyl': {
    'description': "Biphenyl",
    'name': "biphenyl",
    # PBE-gpaw relaxed
    'ionization energy': 8.16,
    'symbols': 'C6H5C6H5',
    'magmoms': None,
    'positions': [[-0.74081, -0.00000, -0.00003],
                  [-1.46261, -1.20370, -0.00993],
                  [-2.85531, -1.20350, -0.00663],
                  [-3.55761, -0.00000, -0.00003],
                  [-2.85531,  1.20350,  0.00667],
                  [-1.46261,  1.20370,  0.00997],
                  [-0.92071, -2.14850,  0.00967],
                  [-3.38981, -2.15110, -0.00083],
                  [-4.64571, -0.00000, -0.00003],
                  [-3.38981,  2.15110,  0.00077],
                  [-0.92071,  2.14850, -0.00963],
                  [ 3.55849, -0.00000, -0.00003],
                  [ 2.85509, -0.86640, -0.83553],
                  [ 1.46289, -0.87000, -0.83153],
                  [ 0.73969, -0.00000, -0.00003],
                  [ 1.46289,  0.87000,  0.83157],
                  [ 2.85509,  0.86640,  0.83547],
                  [ 4.64659, -0.00000, -0.00003],
                  [ 3.39189, -1.53770, -1.50253],
                  [ 0.91869, -1.53310, -1.50263],
                  [ 0.91869,  1.53310,  1.50267],
                  [ 3.39189,  1.53770,  1.50257]]},
'C60': {
    'description': "Buckminsterfullerene, I*h symm.",
    'name': "C_{60}",
    # The Buckyball has two degrees of freedom, the C-C bond, and the C=C bond.
    # This is an LDA-gpaw relaxed structure with bond lengths 1.437 and 1.385.
    # Experimentally, the two bond lengths are 1.45 and 1.40 Angstrom.
    'symbols': 'C60',
    'magmoms': None,
    'positions': [[ 2.2101953,  0.5866631,  2.6669504],
                  [ 3.1076393,  0.1577008,  1.6300286],
                  [ 1.3284430, -0.3158939,  3.2363232],
                  [ 3.0908709, -1.1585005,  1.2014240],
                  [ 3.1879245, -1.4574599, -0.1997005],
                  [ 3.2214623,  1.2230966,  0.6739440],
                  [ 3.3161210,  0.9351586, -0.6765151],
                  [ 3.2984981, -0.4301142, -1.1204138],
                  [-0.4480842,  1.3591484,  3.2081020],
                  [ 0.4672056,  2.2949830,  2.6175264],
                  [-0.0256575,  0.0764219,  3.5086259],
                  [ 1.7727917,  1.9176584,  2.3529691],
                  [ 2.3954623,  2.3095689,  1.1189539],
                  [-0.2610195,  3.0820935,  1.6623117],
                  [ 0.3407726,  3.4592388,  0.4745968],
                  [ 1.6951171,  3.0692446,  0.1976623],
                  [-2.1258394, -0.8458853,  2.6700963],
                  [-2.5620990,  0.4855202,  2.3531715],
                  [-0.8781521, -1.0461985,  3.2367302],
                  [-1.7415096,  1.5679963,  2.6197333],
                  [-1.6262468,  2.6357030,  1.6641811],
                  [-3.2984810,  0.4301871,  1.1204208],
                  [-3.1879469,  1.4573895,  0.1996030],
                  [-2.3360261,  2.5813627,  0.4760912],
                  [-0.5005210, -2.9797771,  1.7940308],
                  [-1.7944338, -2.7729087,  1.2047891],
                  [-0.0514245, -2.1328841,  2.7938830],
                  [-2.5891471, -1.7225828,  1.6329715],
                  [-3.3160705, -0.9350636,  0.6765268],
                  [-1.6951919, -3.0692581, -0.1976564],
                  [-2.3954901, -2.3096853, -1.1189862],
                  [-3.2214182, -1.2231835, -0.6739581],
                  [ 2.1758234, -2.0946263,  1.7922529],
                  [ 1.7118619, -2.9749681,  0.7557198],
                  [ 1.3130656, -1.6829416,  2.7943892],
                  [ 0.3959024, -3.4051395,  0.7557638],
                  [-0.3408219, -3.4591883, -0.4745610],
                  [ 2.3360057, -2.5814499, -0.4761050],
                  [ 1.6263757, -2.6357349, -1.6642309],
                  [ 0.2611352, -3.0821271, -1.6622618],
                  [-2.2100844, -0.5868636, -2.6670300],
                  [-1.7726970, -1.9178969, -2.3530466],
                  [-0.4670723, -2.2950509, -2.6175105],
                  [-1.3283500,  0.3157683, -3.2362375],
                  [-2.1759882,  2.0945383, -1.7923294],
                  [-3.0909663,  1.1583472, -1.2015749],
                  [-3.1076090, -0.1578453, -1.6301627],
                  [-1.3131365,  1.6828292, -2.7943639],
                  [ 0.5003224,  2.9799637, -1.7940203],
                  [-0.3961148,  3.4052817, -0.7557272],
                  [-1.7120629,  2.9749122, -0.7557988],
                  [ 0.0512824,  2.1329478, -2.7937450],
                  [ 2.1258630,  0.8460809, -2.6700534],
                  [ 2.5891853,  1.7227742, -1.6329562],
                  [ 1.7943010,  2.7730684, -1.2048262],
                  [ 0.8781323,  1.0463514, -3.2365313],
                  [ 0.4482452, -1.3591061, -3.2080510],
                  [ 1.7416948, -1.5679557, -2.6197714],
                  [ 2.5621724, -0.4853529, -2.3532026],
                  [ 0.0257904, -0.0763567, -3.5084446]]},
'LiH': {
    'description': "Lithium hydride (LiH), C*v symm.",
    'name': "LiH",
    'enthalpy': 33.3,
    'ZPE': 2.0149,
    'thermal correction': 2.0783,
    'ionization energy': 7.90,
    'symbols': 'LiH',
    'magmoms': None,
    'positions': [[ 0.  ,  0.  ,  0.41],
                  [ 0.  ,  0.  , -1.23]]},
'BeH': {
    'description': "Beryllium hydride (BeH), D*h symm.",
    'name': "BeH",
    'enthalpy': 81.7,
    'ZPE': 2.9073,
    'thermal correction': 2.0739,
    'ionization energy': 8.21,
    'symbols': 'BeH',
    'magmoms': [ 1.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.269654],
                  [ 0.      ,  0.      , -1.078616]]},
'CH': {
    'description': "CH radical. Doublet, C*v symm.",
    'name': "CH",
    'enthalpy': 142.5,
    'ZPE': 3.9659,
    'thermal correction': 2.0739,
    'ionization energy': 10.64,
    'symbols': 'CH',
    'magmoms': [ 1.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.160074],
                  [ 0.      ,  0.      , -0.960446]]},
'CH2_s3B1d': {
    'description': "Triplet methylene (CH2), C2v symm, 3-B1.",
    'name': "CH_2 (^3B_1)",
    'enthalpy': 93.7,
    'ZPE': 10.6953,
    'thermal correction': 2.3877,
    'ionization energy': 10.40,
    'symbols': 'CHH',
    'magmoms': [ 2.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.110381],
                  [ 0.      ,  0.982622, -0.331142],
                  [ 0.      , -0.982622, -0.331142]]},
'CH2_s1A1d': {
    'description': "Singlet methylene (CH2), C2v symm, 1-A1.",
    'name': "CH_2 (^1A_1)",
    'enthalpy': 102.8,
    'ZPE': 10.2422,
    'thermal correction': 2.3745,
    'symbols': 'CHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.174343],
                  [ 0.      ,  0.862232, -0.523029],
                  [ 0.      , -0.862232, -0.523029]]},
'CH3': {
    'description': "Methyl radical (CH3), D3h symm.",
    'name': "CH_3",
    'enthalpy': 35.0,
    'ZPE': 18.3383,
    'thermal correction': 2.5383,
    'ionization energy': 9.84,
    'symbols': 'CHHH',
    'magmoms': [ 1.,  0.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  1.07841 ,  0.      ],
                  [ 0.93393 , -0.539205,  0.      ],
                  [-0.93393 , -0.539205,  0.      ]]},
'CH4': {
    'description': "Methane (CH4), Td symm.",
    'name': "CH_4",
    'enthalpy': -17.9,
    'ZPE': 27.6744,
    'thermal correction': 2.3939,
    'ionization energy': 12.64,
    'vertical ionization energy': 13.60,
    'symbols': 'CHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.629118,  0.629118,  0.629118],
                  [-0.629118, -0.629118,  0.629118],
                  [ 0.629118, -0.629118, -0.629118],
                  [-0.629118,  0.629118, -0.629118]]},
'NH': {
    'description': "NH, triplet, C*v symm.",
    'name': "NH",
    'enthalpy': 85.2,
    'ZPE': 4.5739,
    'thermal correction': 2.0739,
    'ionization energy': 13.10,
    'vertical ionization energy': 13.49,
    'symbols': 'NH',
    'magmoms': [ 2.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.129929],
                  [ 0.      ,  0.      , -0.909501]]},
'NH2': {
    'description': "NH2 radical, C2v symm, 2-B1.",
    'name': "NH_2",
    'enthalpy': 45.1,
    'ZPE': 11.7420,
    'thermal correction': 2.3726,
    'ionization energy': 10.78,
    'vertical ionization energy': 12.00,
    'symbols': 'NHH',
    'magmoms': [ 1.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.14169 ],
                  [ 0.      ,  0.806442, -0.495913],
                  [ 0.      , -0.806442, -0.495913]]},
'NH3': {
    'description': "Ammonia (NH3), C3v symm.",
    'name': "NH_3",
    'enthalpy': -11.0,
    'ZPE': 21.2462,
    'thermal correction': 2.3896,
    'ionization energy': 10.07,
    'vertical ionization energy': 10.82,
    'symbols': 'NHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.116489],
                  [ 0.      ,  0.939731, -0.271808],
                  [ 0.813831, -0.469865, -0.271808],
                  [-0.813831, -0.469865, -0.271808]]},
'OH': {
    'description': "OH radical, C*v symm.",
    'name': "OH",
    'enthalpy': 9.4,
    'ZPE': 5.2039,
    'thermal correction': 2.0739,
    'ionization energy': 13.02,
    'symbols': 'OH',
    'magmoms': [ 0.5,  0.5],
    'positions': [[ 0.      ,  0.      ,  0.108786],
                  [ 0.      ,  0.      , -0.870284]]},
'H2O': {
    'description': "Water (H2O), C2v symm.",
    'name': "H_2O",
    'enthalpy': -57.8,
    'ZPE': 13.2179,
    'thermal correction': 2.3720,
    'ionization energy': 12.62,
    'symbols': 'OHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.119262],
                  [ 0.      ,  0.763239, -0.477047],
                  [ 0.      , -0.763239, -0.477047]]},
'HF': {
    'description': "Hydrogen fluoride (HF), C*v symm.",
    'name': "HF",
    'enthalpy': -65.1,
    'ZPE': 5.7994,
    'thermal correction': 2.0733,
    'ionization energy': 16.03,
    'vertical ionization energy': 16.12,
    'symbols': 'FH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.093389],
                  [ 0.      ,  0.      , -0.840502]]},
'SiH2_s1A1d': {
    'description': "Singlet silylene (SiH2), C2v symm, 1-A1.",
    'name': "SiH_2 (^1A_1)",
    'enthalpy': 65.2,
    'ZPE': 7.1875,
    'thermal correction': 2.3927,
    'ionization energy': 8.92,
    'symbols': 'SiHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.131272],
                  [ 0.      ,  1.096938, -0.918905],
                  [ 0.      , -1.096938, -0.918905]]},
'SiH2_s3B1d': {
    'description': "Triplet silylene (SiH2), C2v symm, 3-B1.",
    'name': "SiH_2 (^3B_1)",
    'enthalpy': 86.2,
    'ZPE': 7.4203,
    'thermal correction': 2.4078,
    'symbols': 'SiHH',
    'magmoms': [ 2.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.094869],
                  [ 0.      ,  1.271862, -0.664083],
                  [ 0.      , -1.271862, -0.664083]]},
'SiH3': {
    'description': "Silyl radical (SiH3), C3v symm.",
    'name': "SiH_3",
    'enthalpy': 47.9,
    'ZPE': 13.0898,
    'thermal correction': 2.4912,
    'ionization energy': 8.14,
    'vertical ionization energy': 8.74,
    'symbols': 'SiHHH',
    'magmoms': [ 1.,  0.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.079299],
                  [ 0.      ,  1.41328 , -0.370061],
                  [ 1.223937, -0.70664 , -0.370061],
                  [-1.223937, -0.70664 , -0.370061]]},
'SiH4': {
    'description': "Silane (SiH4), Td symm.",
    'name': "SiH_4",
    'enthalpy': 8.2,
    'ZPE': 19.2664,
    'thermal correction': 2.5232,
    'ionization energy': 11.00,
    'vertical ionization energy': 12.30,
    'symbols': 'SiHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.856135,  0.856135,  0.856135],
                  [-0.856135, -0.856135,  0.856135],
                  [-0.856135,  0.856135, -0.856135],
                  [ 0.856135, -0.856135, -0.856135]]},
'PH2': {
    'description': "PH2 radical, C2v symm.",
    'name': "PH_2",
    'enthalpy': 33.1,
    'ZPE': 8.2725,
    'thermal correction': 2.3845,
    'ionization energy': 9.82,
    'symbols': 'PHH',
    'magmoms': [ 1.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.115396],
                  [ 0.      ,  1.025642, -0.865468],
                  [ 0.      , -1.025642, -0.865468]]},
'PH3': {
    'description': "Phosphine (PH3), C3v symm.",
    'name': "PH_3",
    'enthalpy': 1.3,
    'ZPE': 14.7885,
    'thermal correction': 2.4203,
    'ionization energy': 9.87,
    'vertical ionization energy': 10.95,
    'symbols': 'PHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.124619],
                  [ 0.      ,  1.200647, -0.623095],
                  [ 1.039791, -0.600323, -0.623095],
                  [-1.039791, -0.600323, -0.623095]]},
'SH2': {
    'description': "Hydrogen sulfide (H2S), C2v symm.",
    'name': "SH_2",
    'enthalpy': -4.9,
    'ZPE': 9.3129,
    'thermal correction': 2.3808,
    'ionization energy': 10.46,
    'vertical ionization energy': 10.50,
    'symbols': 'SHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.102135],
                  [ 0.      ,  0.974269, -0.817083],
                  [ 0.      , -0.974269, -0.817083]]},
'HCl': {
    'description': "Hydrogen chloride (HCl), C*v symm.",
    'name': "HCl",
    'enthalpy': -22.1,
    'ZPE': 4.1673,
    'thermal correction': 2.0739,
    'ionization energy': 12.74,
    'symbols': 'ClH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.07111 ],
                  [ 0.      ,  0.      , -1.208868]]},
'Li2': {
    'description': "Dilithium (Li2), D*h symm.",
    'name': "Li_2",
    'enthalpy': 51.6,
    'ZPE': 0.4838,
    'thermal correction': 2.3086,
    'ionization energy': 5.11,
    'symbols': 'LiLi',
    'magmoms': None,
    'positions': [[ 0.     ,  0.     ,  1.38653],
                  [ 0.     ,  0.     , -1.38653]]},
'LiF': {
    'description': "Lithium Fluoride (LiF), C*v symm.",
    'name': "LiF",
    'enthalpy': -80.1,
    'ZPE': 1.4019,
    'thermal correction': 2.0990,
    'ionization energy': 11.30,
    'symbols': 'LiF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.174965],
                  [ 0.      ,  0.      ,  0.391655]]},
'C2H2': {
    'description': "Acetylene (C2H2), D*h symm.",
    'name': "C_2H_2",
    'enthalpy': 54.2,
    'ZPE': 16.6001,
    'thermal correction': 2.4228,
    'ionization energy': 11.40,
    'vertical ionization energy': 11.49,
    'symbols': 'CCHH',
    'magmoms': None,
    'positions': [[ 0.     ,  0.     ,  0.60808],
                  [ 0.     ,  0.     , -0.60808],
                  [ 0.     ,  0.     , -1.67399],
                  [ 0.     ,  0.     ,  1.67399]]},
'C2H4': {
    'description': "Ethylene (H2C=CH2), D2h symm.",
    'name': "C_2H_4",
    'enthalpy': 12.5,
    'ZPE': 31.5267,
    'thermal correction': 2.5100,
    'ionization energy': 11.40,
    'vertical ionization energy': 11.49,
    'symbols': 'CCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.66748 ],
                  [ 0.      ,  0.      , -0.66748 ],
                  [ 0.      ,  0.922832,  1.237695],
                  [ 0.      , -0.922832,  1.237695],
                  [ 0.      ,  0.922832, -1.237695],
                  [ 0.      , -0.922832, -1.237695]]},
'C2H6': {
    'description': "Ethane (H3C-CH3), D3d symm.",
    'name': "C_2H_6",
    'enthalpy': -20.1,
    'ZPE': 46.0950,
    'thermal correction': 2.7912,
    'symbols': 'CCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.762209],
                  [ 0.      ,  0.      , -0.762209],
                  [ 0.      ,  1.018957,  1.157229],
                  [-0.882443, -0.509479,  1.157229],
                  [ 0.882443, -0.509479,  1.157229],
                  [ 0.      , -1.018957, -1.157229],
                  [-0.882443,  0.509479, -1.157229],
                  [ 0.882443,  0.509479, -1.157229]]},
'CN': {
    'description': "Cyano radical (CN), C*v symm, 2-Sigma+.",
    'name': "CN",
    'enthalpy': 104.9,
    'ZPE': 3.0183,
    'thermal correction': 2.0739,
    'ionization energy': 13.60,
    'symbols': 'CN',
    'magmoms': [ 1.,  0.],
    'positions': [[ 0.      ,  0.      , -0.611046],
                  [ 0.      ,  0.      ,  0.523753]]},
'HCN': {
    'description': "Hydrogen cyanide (HCN), C*v symm.",
    'name': "HCN",
    'enthalpy': 31.5,
    'ZPE': 10.2654,
    'thermal correction': 2.1768,
    'ionization energy': 13.60,
    'vertical ionization energy': 13.61,
    'symbols': 'CNH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -0.511747],
                  [ 0.      ,  0.      ,  0.664461],
                  [ 0.      ,  0.      , -1.580746]]},
'CO': {
    'description': "Carbon monoxide (CO), C*v symm.",
    'name': "CO",
    'enthalpy': -26.4,
    'ZPE': 3.1062,
    'thermal correction': 2.0739,
    'ionization energy': 14.01,
    'vertical ionization energy': 14.01,
    'symbols': 'OC',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.493003],
                  [ 0.      ,  0.      , -0.657337]]},
'HCO': {
    'description': "HCO radical, Bent Cs symm.",
    'name': "HCO",
    'enthalpy': 10.0,
    'ZPE': 8.0290,
    'thermal correction': 2.3864,
    'ionization energy': 8.12,
    'vertical ionization energy': 9.31,
    'symbols': 'COH',
    'magmoms': [ 1.,  0.,  0.],
    'positions': [[ 0.06256 ,  0.593926,  0.      ],
                  [ 0.06256 , -0.596914,  0.      ],
                  [-0.875835,  1.211755,  0.      ]]},
'H2CO': {
    'description': "Formaldehyde (H2C=O), C2v symm.",
    'name': "H_2CO",
    'enthalpy': -26.0,
    'ZPE': 16.4502,
    'thermal correction': 2.3927,
    'ionization energy': 10.88,
    'vertical ionization energy': 10.88,
    'symbols': 'OCHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.683501],
                  [ 0.      ,  0.      , -0.536614],
                  [ 0.      ,  0.93439 , -1.124164],
                  [ 0.      , -0.93439 , -1.124164]]},
'CH3OH': {
    'description': "Methanol (CH3-OH), Cs symm.",
    'name': "H_3COH",
    'enthalpy': -48.0,
    'ZPE': 31.6635,
    'thermal correction': 2.6832,
    'ionization energy': 10.84,
    'vertical ionization energy': 10.96,
    'symbols': 'COHHHH',
    'magmoms': None,
    'positions': [[-0.047131,  0.664389,  0.      ],
                  [-0.047131, -0.758551,  0.      ],
                  [-1.092995,  0.969785,  0.      ],
                  [ 0.878534, -1.048458,  0.      ],
                  [ 0.437145,  1.080376,  0.891772],
                  [ 0.437145,  1.080376, -0.891772]]},
'N2': {
    'description': "N2 molecule, D*h symm.",
    'name': "N_2",
    'enthalpy': 0.0,
    'ZPE': 3.4243,
    'thermal correction': 2.0733,
    'ionization energy': 15.58,
    'vertical ionization energy': 15.58,
    'symbols': 'NN',
    'magmoms': None,
    'positions': [[ 0.     ,  0.     ,  0.56499],
                  [ 0.     ,  0.     , -0.56499]]},
'N2H4': {
    'description': "Hydrazine (H2N-NH2), C2 symm.",
    'name': "H_2NNH_2",
    'enthalpy': 22.8,
    'ZPE': 32.9706,
    'thermal correction': 2.6531,
    'ionization energy': 8.10,
    'vertical ionization energy': 8.98,
    'symbols': 'NNHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.718959, -0.077687],
                  [ 0.      , -0.718959, -0.077687],
                  [ 0.211082,  1.092752,  0.847887],
                  [-0.948214,  1.005026, -0.304078],
                  [-0.211082, -1.092752,  0.847887],
                  [ 0.948214, -1.005026, -0.304078]]},
'NO': {
    'description': "NO radical, C*v symm, 2-Pi.",
    'name': "NO",
    'enthalpy': 21.6,
    'ZPE': 2.7974,
    'thermal correction': 2.0745,
    'ionization energy': 9.26,
    'vertical ionization energy': 9.26,
    'symbols': 'NO',
    'magmoms': [ 0.6,  0.4],
    'positions': [[ 0.      ,  0.      , -0.609442],
                  [ 0.      ,  0.      ,  0.533261]]},
'O2': {
    'description': "O2 molecule, D*h symm, Triplet.",
    'name': "O_2",
    'enthalpy': 0.0,
    'ZPE': 2.3444,
    'thermal correction': 2.0752,
    'ionization energy': 12.07,
    'vertical ionization energy': 12.30,
    'symbols': 'OO',
    'magmoms': [ 1.,  1.],
    'positions': [[ 0.      ,  0.      ,  0.622978],
                  [ 0.      ,  0.      , -0.622978]]},
'H2O2': {
    'description': "Hydrogen peroxide (HO-OH), C2 symm.",
    'name': "HOOH",
    'enthalpy': -32.5,
    'ZPE': 16.4081,
    'thermal correction': 2.6230,
    'ionization energy': 10.58,
    'vertical ionization energy': 11.70,
    'symbols': 'OOHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.734058, -0.05275 ],
                  [ 0.      , -0.734058, -0.05275 ],
                  [ 0.839547,  0.880752,  0.422001],
                  [-0.839547, -0.880752,  0.422001]]},
'F2': {
    'description': "F2 molecule, D*h symm.",
    'name': "F_2",
    'enthalpy': 0.0,
    'ZPE': 1.5179,
    'thermal correction': 2.0915,
    'ionization energy': 15.70,
    'vertical ionization energy': 15.70,
    'symbols': 'FF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.710304],
                  [ 0.      ,  0.      , -0.710304]]},
'CO2': {
    'description': "Carbon dioxide (CO2), D*h symm.",
    'name': "CO_2",
    'enthalpy': -94.1,
    'ZPE': 7.3130,
    'thermal correction': 2.2321,
    'ionization energy': 13.78,
    'vertical ionization energy': 13.78,
    'symbols': 'COO',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  0.      ,  1.178658],
                  [ 0.      ,  0.      , -1.178658]]},
'Na2': {
    'description': "Disodium (Na2), D*h symm.",
    'name': "Na_2",
    'enthalpy': 34.0,
    'ZPE': 0.2246,
    'thermal correction': 2.4699,
    'ionization energy': 4.89,
    'symbols': 'NaNa',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.576262],
                  [ 0.      ,  0.      , -1.576262]]},
'Si2': {
    'description': "Si2 molecule, D*h symm, Triplet (3-Sigma-G-).",
    'name': "Si_2",
    'enthalpy': 139.9,
    'ZPE': 0.7028,
    'thermal correction': 2.2182,
    'ionization energy': 7.90,
    'symbols': 'SiSi',
    'magmoms': [ 1.,  1.],
    'positions': [[ 0.      ,  0.      ,  1.130054],
                  [ 0.      ,  0.      , -1.130054]]},
'P2': {
    'description': "P2 molecule, D*h symm.",
    'name': "P_2",
    'enthalpy': 34.3,
    'ZPE': 1.1358,
    'thermal correction': 2.1235,
    'ionization energy': 10.53,
    'vertical ionization energy': 10.62,
    'symbols': 'PP',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.966144],
                  [ 0.      ,  0.      , -0.966144]]},
'S2': {
    'description': "S2 molecule, D*h symm, triplet.",
    'name': "S_2",
    'enthalpy': 30.7,
    'ZPE': 1.0078,
    'thermal correction': 2.1436,
    'ionization energy': 9.36,
    'vertical ionization energy': 9.55,
    'symbols': 'SS',
    'magmoms': [ 1.,  1.],
    'positions': [[ 0.      ,  0.      ,  0.960113],
                  [ 0.      ,  0.      , -0.960113]]},
'Cl2': {
    'description': "Cl2 molecule, D*h symm.",
    'name': "Cl_2",
    'enthalpy': 0.0,
    'ZPE': 0.7737,
    'thermal correction': 2.1963,
    'ionization energy': 11.48,
    'vertical ionization energy': 11.49,
    'symbols': 'ClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.007541],
                  [ 0.      ,  0.      , -1.007541]]},
'NaCl': {
    'description': "Sodium Chloride (NaCl), C*v symm.",
    'name': "NaCl",
    'enthalpy': -43.6,
    'ZPE': 0.5152,
    'thermal correction': 2.2935,
    'ionization energy': 9.20,
    'vertical ionization energy': 9.80,
    'symbols': 'NaCl',
    'magmoms': None,
    'positions': [[ 0.     ,  0.     , -1.45166],
                  [ 0.     ,  0.     ,  0.93931]]},
'SiO': {
    'description': "Silicon monoxide (SiO), C*v symm.",
    'name': "SiO",
    'enthalpy': -24.6,
    'ZPE': 1.7859,
    'thermal correction': 2.0821,
    'ionization energy': 11.49,
    'symbols': 'SiO',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.560846],
                  [ 0.      ,  0.      , -0.98148 ]]},
'CS': {
    'description': "Carbon monosulfide (CS), C*v symm.",
    'name': "SC",
    'enthalpy': 66.9,
    'ZPE': 1.8242,
    'thermal correction': 2.0814,
    'ionization energy': 11.33,
    'symbols': 'CS',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.123382],
                  [ 0.      ,  0.      ,  0.421268]]},
'SO': {
    'description': "Sulfur monoxide (SO), C*v symm, triplet.",
    'name': "SO",
    'enthalpy': 1.2,
    'ZPE': 1.6158,
    'thermal correction': 2.0877,
    'ionization energy': 11.29,
    'symbols': 'OS',
    'magmoms': [ 1.,  1.],
    'positions': [[ 0.      ,  0.      , -1.015992],
                  [ 0.      ,  0.      ,  0.507996]]},
'ClO': {
    'description': "ClO radical, C*v symm, 2-PI.",
    'name': "ClO",
    'enthalpy': 24.2,
    'ZPE': 1.1923,
    'thermal correction': 2.1172,
    'ionization energy': 10.89,
    'vertical ionization energy': 11.01,
    'symbols': 'ClO',
    'magmoms': [ 1.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.514172],
                  [ 0.      ,  0.      , -1.092615]]},
'ClF': {
    'description': "ClF molecule, C*v symm, 1-SG.",
    'name': "FCl",
    'enthalpy': -13.2,
    'ZPE': 1.1113,
    'thermal correction': 2.1273,
    'ionization energy': 12.66,
    'vertical ionization energy': 12.77,
    'symbols': 'FCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.084794],
                  [ 0.      ,  0.      ,  0.574302]]},
'Si2H6': {
    'description': "Disilane (H3Si-SiH3), D3d symm.",
    'name': "Si_2H_6",
    'enthalpy': 19.1,
    'ZPE': 30.2265,
    'thermal correction': 3.7927,
    'ionization energy': 9.74,
    'vertical ionization energy': 10.53,
    'symbols': 'SiSiHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.167683],
                  [ 0.      ,  0.      , -1.167683],
                  [ 0.      ,  1.393286,  1.68602 ],
                  [-1.206621, -0.696643,  1.68602 ],
                  [ 1.206621, -0.696643,  1.68602 ],
                  [ 0.      , -1.393286, -1.68602 ],
                  [-1.206621,  0.696643, -1.68602 ],
                  [ 1.206621,  0.696643, -1.68602 ]]},
'CH3Cl': {
    'description': "Methyl chloride (CH3Cl), C3v symm.",
    'name': "CH_3Cl",
    'enthalpy': -19.6,
    'ZPE': 23.3013,
    'thermal correction': 2.4956,
    'symbols': 'CClHHH',
    'ionization energy': 11.26,
    'vertical ionization energy': 11.29,
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.121389],
                  [ 0.      ,  0.      ,  0.655951],
                  [ 0.      ,  1.029318, -1.47428 ],
                  [ 0.891415, -0.514659, -1.47428 ],
                  [-0.891415, -0.514659, -1.47428 ]]},
'CH3SH': {
    'description': "Methanethiol (H3C-SH), Staggered, Cs symm.",
    'name': "H_3CSH",
    'enthalpy': -5.5,
    'ZPE': 28.3973,
    'thermal correction': 2.8690,
    'ionization energy': 9.44,
    'vertical ionization energy': 9.44,
    'symbols': 'CSHHHH',
    'magmoms': None,
    'positions': [[-0.047953,  1.149519,  0.      ],
                  [-0.047953, -0.664856,  0.      ],
                  [ 1.283076, -0.823249,  0.      ],
                  [-1.092601,  1.461428,  0.      ],
                  [ 0.432249,  1.551207,  0.892259],
                  [ 0.432249,  1.551207, -0.892259]]},
'HOCl': {
    'description': "HOCl molecule, Cs symm.",
    'name': "HOCl",
    'enthalpy': -17.8,
    'ZPE': 8.1539,
    'thermal correction': 2.4416,
    'ionization energy': 11.12,
    'symbols': 'OHCl',
    'magmoms': None,
    'positions': [[ 0.036702,  1.113517,  0.      ],
                  [-0.917548,  1.328879,  0.      ],
                  [ 0.036702, -0.602177,  0.      ]]},
'SO2': {
    'description': "Sulfur dioxide (SO2), C2v symm.",
    'name': "SO_2",
    'enthalpy': -71.0,
    'ZPE': 4.3242,
    'thermal correction': 2.5245,
    'ionization energy': 12.35,
    'vertical ionization energy': 12.50,
    'symbols': 'SOO',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.370268],
                  [ 0.      ,  1.277617, -0.370268],
                  [ 0.      , -1.277617, -0.370268]]},
'BF3': {
    'description': "BF3, Planar D3h symm.",
    'name': "BF_3",
    'enthalpy': -271.4,
    'ZPE': 7.8257,
    'thermal correction': 2.7893,
    'symbols': 'BFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  1.32176 ,  0.      ],
                  [ 1.144678, -0.66088 ,  0.      ],
                  [-1.144678, -0.66088 ,  0.      ]]},
'BCl3': {
    'description': "BCl3, Planar D3h symm.",
    'name': "BCl_3",
    'enthalpy': -96.3,
    'ZPE': 4.6536,
    'thermal correction': 3.3729,
    'symbols': 'BClClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  1.735352,  0.      ],
                  [ 1.502859, -0.867676,  0.      ],
                  [-1.502859, -0.867676,  0.      ]]},
'AlF3': {
    'description': "AlF3, Planar D3h symm.",
    'name': "AlF_3",
    'enthalpy': -289.0,
    'ZPE': 4.8645,
    'thermal correction': 3.3986,
    'symbols': 'AlFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  1.64472 ,  0.      ],
                  [ 1.424369, -0.82236 ,  0.      ],
                  [-1.424369, -0.82236 ,  0.      ]]},
'AlCl3': {
    'description': "AlCl3, Planar D3h symm.",
    'name': "AlCl_3",
    'enthalpy': -139.7,
    'ZPE': 2.9687,
    'thermal correction': 3.9464,
    'symbols': 'AlClClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  2.069041,  0.      ],
                  [ 1.791842, -1.03452 ,  0.      ],
                  [-1.791842, -1.03452 ,  0.      ]]},
'CF4': {
    'description': "CF4, Td symm.",
    'name': "CF_4",
    'enthalpy': -223.0,
    'ZPE': 10.5999,
    'thermal correction': 3.0717,
    'symbols': 'CFFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.767436,  0.767436,  0.767436],
                  [-0.767436, -0.767436,  0.767436],
                  [-0.767436,  0.767436, -0.767436],
                  [ 0.767436, -0.767436, -0.767436]]},
'CCl4': {
    'description': "CCl4, Td symm.",
    'name': "CCl_4",
    'enthalpy': -22.9,
    'ZPE': 5.7455,
    'thermal correction': 4.1754,
    'symbols': 'CClClClCl',
    'magmoms': None,
    'positions': [[ 0.     ,  0.     ,  0.     ],
                  [ 1.02134,  1.02134,  1.02134],
                  [-1.02134, -1.02134,  1.02134],
                  [-1.02134,  1.02134, -1.02134],
                  [ 1.02134, -1.02134, -1.02134]]},
'OCS': {
    'description': "O=C=S, Linear, C*v symm.",
    'name': "COS",
    'enthalpy': -33.1,
    'ZPE': 5.7706,
    'thermal correction': 2.3663,
    'symbols': 'OCS',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.699243],
                  [ 0.      ,  0.      , -0.520492],
                  [ 0.      ,  0.      ,  1.044806]]},
'CS2': {
    'description': "CS2, Linear, D*h symm.",
    'name': "CS_2",
    'enthalpy': 28.0,
    'ZPE': 4.3380,
    'thermal correction': 2.5326,
    'symbols': 'SCS',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.561117],
                  [ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  0.      , -1.561117]]},
'COF2': {
    'description': "COF2, C2v symm.",
    'name': "COF_2",
    'enthalpy': -149.1,
    'ZPE': 8.8215,
    'thermal correction': 2.6619,
    'symbols': 'OCFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.330715],
                  [ 0.      ,  0.      ,  0.144358],
                  [ 0.      ,  1.06949 , -0.639548],
                  [ 0.      , -1.06949 , -0.639548]]},
'SiF4': {
    'description': "SiF4, Td symm.",
    'name': "SiF_4",
    'enthalpy': -386.0,
    'ZPE': 7.8771,
    'thermal correction': 3.7054,
    'symbols': 'SiFFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.912806,  0.912806,  0.912806],
                  [-0.912806, -0.912806,  0.912806],
                  [-0.912806,  0.912806, -0.912806],
                  [ 0.912806, -0.912806, -0.912806]]},
'SiCl4': {
    'description': "SiCl4, Td symm.",
    'name': "SiCl_4",
    'enthalpy': -158.4,
    'ZPE': 4.4396,
    'thermal correction': 4.7182,
    'symbols': 'SiClClClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 1.169349,  1.169349,  1.169349],
                  [-1.169349, -1.169349,  1.169349],
                  [ 1.169349, -1.169349, -1.169349],
                  [-1.169349,  1.169349, -1.169349]]},
'N2O': {
    'description': "N2O, Cs symm.",
    'name': "N_2O",
    'enthalpy': 19.6,
    'ZPE': 6.9748,
    'thermal correction': 2.2710,
    'symbols': 'NNO',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.231969],
                  [ 0.      ,  0.      , -0.060851],
                  [ 0.      ,  0.      ,  1.131218]]},
'ClNO': {
    'description': "ClNO, Cs symm.",
    'name': "ClNO",
    'enthalpy': 12.4,
    'ZPE': 4.0619,
    'thermal correction': 2.7039,
    'symbols': 'ClNO',
    'magmoms': None,
    'positions': [[-0.537724, -0.961291,  0.      ],
                  [ 0.      ,  0.997037,  0.      ],
                  [ 1.142664,  1.170335,  0.      ]]},
'NF3': {
    'description': "NF3, C3v symm.",
    'name': "NF_3",
    'enthalpy': -31.6,
    'ZPE': 6.4477,
    'thermal correction': 2.8301,
    'symbols': 'NFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.489672],
                  [ 0.      ,  1.238218, -0.126952],
                  [ 1.072328, -0.619109, -0.126952],
                  [-1.072328, -0.619109, -0.126952]]},
'PF3': {
    'description': "PF3, C3v symm.",
    'name': "PF_3",
    'enthalpy': -229.1,
    'ZPE': 5.2981,
    'thermal correction': 3.1288,
    'symbols': 'PFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.506767],
                  [ 0.      ,  1.383861, -0.281537],
                  [ 1.198459, -0.691931, -0.281537],
                  [-1.198459, -0.691931, -0.281537]]},
'O3': {
    'description': "O3 (Ozone), C2v symm.",
    'name': "O_3",
    'enthalpy': 34.1,
    'ZPE': 4.6178,
    'thermal correction': 2.4479,
    'symbols': 'OOO',
    'magmoms': None,
    'positions': [[ 0.      ,  1.10381 , -0.228542],
                  [ 0.      ,  0.      ,  0.457084],
                  [ 0.      , -1.10381 , -0.228542]]},
'F2O': {
    'description': "F2O, C2v symm.",
    'name': "F_2O",
    'enthalpy': 5.9,
    'ZPE': 3.4362,
    'thermal correction': 2.5747,
    'symbols': 'FOF',
    'magmoms': None,
    'positions': [[ 0.      ,  1.110576, -0.273729],
                  [ 0.      ,  0.      ,  0.61589 ],
                  [ 0.      , -1.110576, -0.273729]]},
'ClF3': {
    'description': "ClF3, C2v symm.",
    'name': "ClF_3",
    'enthalpy': -38.0,
    'ZPE': 4.2922,
    'thermal correction': 3.3289,
    'symbols': 'ClFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.376796],
                  [ 0.      ,  0.      , -1.258346],
                  [ 0.      ,  1.714544,  0.27331 ],
                  [ 0.      , -1.714544,  0.27331 ]]},
'C2F4': {
    'description': "C2F4 (F2C=CF2), D2H symm.",
    'name': "C_2F_4",
    'enthalpy': -157.4,
    'ZPE': 13.4118,
    'thermal correction': 3.9037,
    'symbols': 'CCFFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.66323 ],
                  [ 0.      ,  0.      , -0.66323 ],
                  [ 0.      ,  1.112665,  1.385652],
                  [ 0.      , -1.112665,  1.385652],
                  [ 0.      ,  1.112665, -1.385652],
                  [ 0.      , -1.112665, -1.385652]]},
'C2Cl4': {
    'description': "C2Cl4 (Cl2C=CCl2), D2h symm.",
    'name': "C_2Cl_4",
    'enthalpy': -3.0,
    'ZPE': 9.4628,
    'thermal correction': 4.7132,
    'symbols': 'CCClClClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.675402],
                  [ 0.      ,  0.      , -0.675402],
                  [ 0.      ,  1.448939,  1.589701],
                  [ 0.      , -1.448939,  1.589701],
                  [ 0.      , -1.448939, -1.589701],
                  [ 0.      ,  1.448939, -1.589701]]},
'CF3CN': {
    'description': "CF3CN, C3v symm.",
    'name': "CF_3CN",
    'enthalpy': -118.4,
    'ZPE': 14.1020,
    'thermal correction': 3.7996,
    'symbols': 'CCFFFN',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -0.32635 ],
                  [ 0.      ,  0.      ,  1.15083 ],
                  [ 0.      ,  1.257579, -0.787225],
                  [ 1.089096, -0.62879 , -0.787225],
                  [-1.089096, -0.62879 , -0.787225],
                  [ 0.      ,  0.      ,  2.329741]]},
'C3H4_C3v': {
    'description': "Propyne (C3H4), C3v symm.",
    'name': "CH_3CCH (propyne)",
    'enthalpy': 44.2,
    'ZPE': 34.2614,
    'thermal correction': 3.1193,
    'symbols': 'CCCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.214947],
                  [ 0.      ,  0.      ,  1.43313 ],
                  [ 0.      ,  0.      , -1.246476],
                  [ 0.      ,  0.      ,  2.498887],
                  [ 0.      ,  1.021145, -1.636167],
                  [ 0.884337, -0.510572, -1.636167],
                  [-0.884337, -0.510572, -1.636167]]},
'C3H4_D2d': {
    'description': "Allene (C3H4), D2d symm.",
    'name': "CH_2=C=CH_2 (allene)",
    'enthalpy': 45.5,
    'ZPE': 34.1189,
    'thermal correction': 2.9744,
    'symbols': 'CCCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  0.      ,  1.31119 ],
                  [ 0.      ,  0.      , -1.31119 ],
                  [ 0.      ,  0.926778,  1.876642],
                  [ 0.      , -0.926778,  1.876642],
                  [ 0.926778,  0.      , -1.876642],
                  [-0.926778,  0.      , -1.876642]]},
'C3H4_C2v': {
    'description': "Cyclopropene (C3H4), C2v symm.",
    'name': "C_3H_4 (cyclopropene)",
    'enthalpy': 66.2,
    'ZPE': 34.7603,
    'thermal correction': 2.6763,
    'symbols': 'CCCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.858299],
                  [ 0.      , -0.650545, -0.498802],
                  [ 0.      ,  0.650545, -0.498802],
                  [ 0.912438,  0.      ,  1.456387],
                  [-0.912438,  0.      ,  1.456387],
                  [ 0.      , -1.584098, -1.038469],
                  [ 0.      ,  1.584098, -1.038469]]},
'C3H6_Cs': {
    'description': "Propene (C3H6), Cs symm.",
    'name': "CH_3CH=CH_2 (propylene)",
    'enthalpy': 4.8,
    'ZPE': 49.1836,
    'thermal correction': 3.1727,
    'symbols': 'CCHHHCHHH',
    'magmoms': None,
    'positions': [[ 1.29129 ,  0.133682,  0.      ],
                  [ 0.      ,  0.479159,  0.      ],
                  [ 1.60116 , -0.90742 ,  0.      ],
                  [ 2.0808  ,  0.877337,  0.      ],
                  [-0.263221,  1.536098,  0.      ],
                  [-1.139757, -0.492341,  0.      ],
                  [-0.776859, -1.523291,  0.      ],
                  [-1.77554 , -0.352861,  0.88042 ],
                  [-1.77554 , -0.352861, -0.88042 ]]},
'C3H6_D3h': {
    'description': "Cyclopropane (C3H6), D3h symm.",
    'name': "C_3H_6 (cyclopropane)",
    'enthalpy': 12.7,
    'ZPE': 50.2121,
    'thermal correction': 2.7272,
    'symbols': 'CCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.866998,  0.      ],
                  [ 0.750842, -0.433499,  0.      ],
                  [-0.750842, -0.433499,  0.      ],
                  [ 0.      ,  1.455762,  0.910526],
                  [ 0.      ,  1.455762, -0.910526],
                  [ 1.260727, -0.727881, -0.910526],
                  [ 1.260727, -0.727881,  0.910526],
                  [-1.260727, -0.727881,  0.910526],
                  [-1.260727, -0.727881, -0.910526]]},
'C3H8': {
    'description': "Propane (C3H8), C2v symm.",
    'name': "C_3H_8 (propane)",
    'enthalpy': -25.0,
    'ZPE': 63.8008,
    'thermal correction': 3.4632,
    'symbols': 'CCCHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.587716],
                  [ 0.      ,  1.266857, -0.260186],
                  [ 0.      , -1.266857, -0.260186],
                  [-0.876898,  0.      ,  1.244713],
                  [ 0.876898,  0.      ,  1.244713],
                  [ 0.      ,  2.16615 ,  0.362066],
                  [ 0.      , -2.16615 ,  0.362066],
                  [ 0.883619,  1.304234, -0.904405],
                  [-0.883619,  1.304234, -0.904405],
                  [-0.883619, -1.304234, -0.904405],
                  [ 0.883619, -1.304234, -0.904405]]},
'butadiene': {
    'description': "Trans-1,3-butadiene (C4H6), C2h symm.",
    'name': "CH_2CHCHCH_2 (butadiene)",
    'enthalpy': 26.3,
    'ZPE': 52.6273,
    'thermal correction': 3.5341,
    'symbols': 'CCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.605711,  1.74655 ,  0.      ],
                  [ 0.605711,  0.404083,  0.      ],
                  [-0.605711, -0.404083,  0.      ],
                  [-0.605711, -1.74655 ,  0.      ],
                  [ 1.527617,  2.317443,  0.      ],
                  [-0.321132,  2.313116,  0.      ],
                  [ 1.553503, -0.13364 ,  0.      ],
                  [-1.553503,  0.13364 ,  0.      ],
                  [ 0.321132, -2.313116,  0.      ],
                  [-1.527617, -2.317443,  0.      ]]},
'2-butyne': {
    'description': "Dimethylacetylene (2-butyne, C4H6), D3h symm (eclipsed).",
    'name': "C_4H_6 (2-butyne)",
    'enthalpy': 34.8,
    'ZPE': 51.8731,
    'thermal correction': 4.2344,
    'symbols': 'CCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  2.071955],
                  [ 0.      ,  0.      ,  0.60997 ],
                  [ 0.      ,  0.      , -0.60997 ],
                  [ 0.      ,  0.      , -2.071955],
                  [ 0.      ,  1.020696,  2.464562],
                  [-0.883949, -0.510348,  2.464562],
                  [ 0.883949, -0.510348,  2.464562],
                  [ 0.      ,  1.020696, -2.464562],
                  [ 0.883949, -0.510348, -2.464562],
                  [-0.883949, -0.510348, -2.464562]]},
'methylenecyclopropane': {
    'description': "Methylenecyclopropane (C4H6), C2v symm.",
    'name': "C_4H_6 (methylene cyclopropane)",
    'enthalpy': 47.9,
    'ZPE': 52.6230,
    'thermal correction': 3.2881,
    'symbols': 'CCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.315026],
                  [ 0.      , -0.76792 , -0.932032],
                  [ 0.      ,  0.76792 , -0.932032],
                  [ 0.      ,  0.      ,  1.640027],
                  [-0.912794, -1.271789, -1.239303],
                  [ 0.912794, -1.271789, -1.239303],
                  [ 0.912794,  1.271789, -1.239303],
                  [-0.912794,  1.271789, -1.239303],
                  [ 0.      , -0.926908,  2.20564 ],
                  [ 0.      ,  0.926908,  2.20564 ]]},
'bicyclobutane': {
    'description': "Bicyclo[1.1.0]butane (C4H6), C2v symm.",
    'name': "C_4H_6 (bicyclobutane)",
    'enthalpy': 51.9,
    'ZPE': 53.3527,
    'thermal correction': 2.9637,
    'symbols': 'CCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  1.131343,  0.310424],
                  [ 0.      , -1.131343,  0.310424],
                  [ 0.747952,  0.      , -0.311812],
                  [-0.747952,  0.      , -0.311812],
                  [ 0.      ,  1.237033,  1.397617],
                  [ 0.      ,  2.077375, -0.227668],
                  [ 0.      , -1.237033,  1.397617],
                  [ 0.      , -2.077375, -0.227668],
                  [ 1.41441 ,  0.      , -1.161626],
                  [-1.41441 ,  0.      , -1.161626]]},
'cyclobutene': {
    'description': "Cyclobutene (C4H6), C2v symm.",
    'name': "C_4H_6 (cyclobutene)",
    'enthalpy': 37.4,
    'ZPE': 53.4105,
    'thermal correction': 3.0108,
    'symbols': 'CCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      , -0.672762,  0.811217],
                  [ 0.      ,  0.672762,  0.811217],
                  [ 0.      , -0.78198 , -0.696648],
                  [ 0.      ,  0.78198 , -0.696648],
                  [ 0.      , -1.422393,  1.597763],
                  [ 0.      ,  1.422393,  1.597763],
                  [-0.88931 , -1.239242, -1.142591],
                  [ 0.88931 , -1.239242, -1.142591],
                  [ 0.88931 ,  1.239242, -1.142591],
                  [-0.88931 ,  1.239242, -1.142591]]},
'cyclobutane': {
    'description': "Cyclobutane (C4H8), D2d symm.",
    'name': "C_4H_8 (cyclobutane)",
    'enthalpy': 6.8,
    'ZPE': 68.3314,
    'thermal correction': 3.2310,
    'symbols': 'CCCCHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  1.071142,  0.147626],
                  [ 0.      , -1.071142,  0.147626],
                  [-1.071142,  0.      , -0.147626],
                  [ 1.071142,  0.      , -0.147626],
                  [ 0.      ,  1.986858, -0.450077],
                  [ 0.      ,  1.342921,  1.20752 ],
                  [ 0.      , -1.986858, -0.450077],
                  [ 0.      , -1.342921,  1.20752 ],
                  [-1.986858,  0.      ,  0.450077],
                  [-1.342921,  0.      , -1.20752 ],
                  [ 1.986858,  0.      ,  0.450077],
                  [ 1.342921,  0.      , -1.20752 ]]},
'isobutene': {
    'description': "Isobutene (C4H8), Single bonds trans, C2v symm.",
    'name': "C_4H_8 (isobutene)",
    'enthalpy': -4.0,
    'ZPE': 66.5693,
    'thermal correction': 3.9495,
    'symbols': 'CCHHCHHHCHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.458807],
                  [ 0.      ,  0.      ,  0.119588],
                  [ 0.      ,  0.924302,  2.028409],
                  [ 0.      , -0.924302,  2.028409],
                  [ 0.      ,  1.272683, -0.678803],
                  [ 0.      ,  2.153042, -0.031588],
                  [ 0.880211,  1.323542, -1.329592],
                  [-0.880211,  1.323542, -1.329592],
                  [ 0.      , -1.272683, -0.678803],
                  [ 0.      , -2.153042, -0.031588],
                  [-0.880211, -1.323542, -1.329592],
                  [ 0.880211, -1.323542, -1.329592]]},
'trans-butane': {
    'description': "Trans-butane (C4H10), C2h symm.",
    'name': "C_4H_{10} (trans butane)",
    'enthalpy': -30.0,
    'ZPE': 81.3980,
    'thermal correction': 4.2633,
    'symbols': 'CCCCHHHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.702581,  1.820873,  0.      ],
                  [ 0.702581,  0.296325,  0.      ],
                  [-0.702581, -0.296325,  0.      ],
                  [-0.702581, -1.820873,  0.      ],
                  [ 1.719809,  2.22234 ,  0.      ],
                  [-1.719809, -2.22234 ,  0.      ],
                  [ 0.188154,  2.210362,  0.883614],
                  [ 0.188154,  2.210362, -0.883614],
                  [-0.188154, -2.210362,  0.883614],
                  [-0.188154, -2.210362, -0.883614],
                  [ 1.247707, -0.07266 , -0.877569],
                  [ 1.247707, -0.07266 ,  0.877569],
                  [-1.247707,  0.07266 , -0.877569],
                  [-1.247707,  0.07266 ,  0.877569]]},
'isobutane': {
    'description': "Isobutane (C4H10), C3v symm.",
    'name': "C_4H_{10} (isobutane)",
    'enthalpy': -32.1,
    'ZPE': 81.1050,
    'thermal correction': 4.2282,
    'symbols': 'CHCHHHCHHHCHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.376949],
                  [ 0.      ,  0.      ,  1.475269],
                  [ 0.      ,  1.45029 , -0.096234],
                  [ 0.      ,  1.493997, -1.190847],
                  [-0.885482,  1.984695,  0.261297],
                  [ 0.885482,  1.984695,  0.261297],
                  [ 1.255988, -0.725145, -0.096234],
                  [ 1.293839, -0.746998, -1.190847],
                  [ 2.161537, -0.225498,  0.261297],
                  [ 1.276055, -1.759198,  0.261297],
                  [-1.255988, -0.725145, -0.096234],
                  [-1.293839, -0.746998, -1.190847],
                  [-1.276055, -1.759198,  0.261297],
                  [-2.161537, -0.225498,  0.261297]]},
'C5H8': {
    'description': "Spiropentane (C5H8), D2d symm.",
    'name': "C_5H_8 (spiropentane)",
    'enthalpy': 44.3,
    'ZPE': 70.9964,
    'thermal correction': 3.7149,
    'symbols': 'CCCCCHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.      ],
                  [ 0.      ,  0.762014,  1.265752],
                  [ 0.      , -0.762014,  1.265752],
                  [ 0.762014,  0.      , -1.265752],
                  [-0.762014,  0.      , -1.265752],
                  [-0.914023,  1.265075,  1.56809 ],
                  [ 0.914023,  1.265075,  1.56809 ],
                  [-0.914023, -1.265075,  1.56809 ],
                  [ 0.914023, -1.265075,  1.56809 ],
                  [ 1.265075, -0.914023, -1.56809 ],
                  [ 1.265075,  0.914023, -1.56809 ],
                  [-1.265075, -0.914023, -1.56809 ],
                  [-1.265075,  0.914023, -1.56809 ]]},
'C6H6': {
    'description': "Benzene (C6H6), D6h symm.",
    'name': "C_6H_6 (benzene)",
    'enthalpy': 19.7,
    'ZPE': 61.9252,
    'thermal correction': 3.3886,
    'ionization energy': 9.24,
    'vertical ionization energy': 9.25,
    'symbols': 'CCCCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  1.395248,  0.      ],
                  [ 1.20832 ,  0.697624,  0.      ],
                  [ 1.20832 , -0.697624,  0.      ],
                  [ 0.      , -1.395248,  0.      ],
                  [-1.20832 , -0.697624,  0.      ],
                  [-1.20832 ,  0.697624,  0.      ],
                  [ 0.      ,  2.48236 ,  0.      ],
                  [ 2.149787,  1.24118 ,  0.      ],
                  [ 2.149787, -1.24118 ,  0.      ],
                  [ 0.      , -2.48236 ,  0.      ],
                  [-2.149787, -1.24118 ,  0.      ],
                  [-2.149787,  1.24118 ,  0.      ]]},
'H2CF2': {
    'description': "Difluoromethane (H2CF2), C2v symm.",
    'name': "CH_2F_2",
    'enthalpy': -107.7,
    'ZPE': 20.2767,
    'thermal correction': 2.5552,
    'symbols': 'CFFHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.502903],
                  [ 0.      ,  1.109716, -0.290601],
                  [ 0.      , -1.109716, -0.290601],
                  [-0.908369,  0.      ,  1.106699],
                  [ 0.908369,  0.      ,  1.106699]]},
'HCF3': {
    'description': "Trifluoromethane (HCF3), C3v symm.",
    'name': "CHF_3",
    'enthalpy': -166.6,
    'ZPE': 15.7072,
    'thermal correction': 2.7717,
    'symbols': 'CHFFF',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.341023],
                  [ 0.      ,  0.      ,  1.429485],
                  [ 0.      ,  1.2582  , -0.128727],
                  [ 1.089633, -0.6291  , -0.128727],
                  [-1.089633, -0.6291  , -0.128727]]},
'H2CCl2': {
    'description': "Dichloromethane (H2CCl2), C2v symm.",
    'name': "CH_2Cl_2",
    'enthalpy': -22.8,
    'ZPE': 18.0930,
    'thermal correction': 2.8527,
    'symbols': 'CClClHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.759945],
                  [ 0.      ,  1.4742  , -0.215115],
                  [ 0.      , -1.4742  , -0.215115],
                  [-0.894585,  0.      ,  1.377127],
                  [ 0.894585,  0.      ,  1.377127]]},
'HCCl3': {
    'description': "Chloroform (HCCl3), C3v symm.",
    'name': "CHCl_3",
    'enthalpy': -24.7,
    'ZPE': 12.1975,
    'thermal correction': 3.4262,
    'symbols': 'CHClClCl',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.451679],
                  [ 0.      ,  0.      ,  1.537586],
                  [ 0.      ,  1.681723, -0.083287],
                  [ 1.456415, -0.840862, -0.083287],
                  [-1.456415, -0.840862, -0.083287]]},
'H3CNH2': {
    'description': "Methylamine (H3C-NH2), Cs symm.",
    'name': "CH_3NH_2 (methylamine)",
    'enthalpy': -5.5,
    'ZPE': 39.5595,
    'thermal correction': 2.7428,
    'symbols': 'CNHHHHH',
    'magmoms': None,
    'positions': [[ 0.051736,  0.704422,  0.      ],
                  [ 0.051736, -0.759616,  0.      ],
                  [-0.941735,  1.176192,  0.      ],
                  [-0.458181, -1.099433,  0.81237 ],
                  [-0.458181, -1.099433, -0.81237 ],
                  [ 0.592763,  1.056727,  0.88067 ],
                  [ 0.592763,  1.056727, -0.88067 ]]},
'CH3CN': {
    'description': "Acetonitrile (CH3-CN), C3v symm.",
    'name': "CH_3CN (methyl cyanide)",
    'enthalpy': 18.0,
    'ZPE': 28.0001,
    'thermal correction': 2.8552,
    'symbols': 'CCNHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.18693 ],
                  [ 0.      ,  0.      ,  0.273874],
                  [ 0.      ,  0.      ,  1.452206],
                  [ 0.      ,  1.024986, -1.56237 ],
                  [ 0.887664, -0.512493, -1.56237 ],
                  [-0.887664, -0.512493, -1.56237 ]]},
'CH3NO2': {
    'description': "Nitromethane (CH3-NO2), Cs symm.",
    'name': "CH_3NO_2 (nitromethane)",
    'enthalpy': -17.8,
    'ZPE': 30.7568,
    'thermal correction': 2.7887,
    'symbols': 'CNHHHOO',
    'magmoms': None,
    'positions': [[-0.114282, -1.314565,  0.      ],
                  [ 0.      ,  0.16648 ,  0.      ],
                  [ 0.899565, -1.715256,  0.      ],
                  [-0.640921, -1.607212,  0.904956],
                  [-0.640921, -1.607212, -0.904956],
                  [ 0.066748,  0.728232, -1.103775],
                  [ 0.066748,  0.728232,  1.103775]]},
'CH3ONO': {
    'description': "Methylnitrite (CH3-O-N=O), NOCH trans, ONOC cis, Cs symm.",
    'name': "CH_3ONO (methyl nitrite)",
    'enthalpy': -15.9,
    'ZPE': 29.9523,
    'thermal correction': 3.3641,
    'symbols': 'COHHHNO',
    'magmoms': None,
    'positions': [[-1.316208,  0.309247,  0.      ],
                  [ 0.      ,  0.896852,  0.      ],
                  [-1.985538,  1.166013,  0.      ],
                  [-1.464336, -0.304637,  0.890672],
                  [-1.464336, -0.304637, -0.890672],
                  [ 1.045334, -0.022815,  0.      ],
                  [ 0.686764, -1.178416,  0.      ]]},
'CH3SiH3': {
    'description': "Methylsilane (CH3-SiH3), C3v symm.",
    'name': "CH_3SiH_3 (methyl silane)",
    'enthalpy': -7.0,
    'ZPE': 37.6606,
    'thermal correction': 3.2486,
    'symbols': 'CSiHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.244466],
                  [ 0.      ,  0.      ,  0.635703],
                  [ 0.      , -1.019762, -1.636363],
                  [-0.88314 ,  0.509881, -1.636363],
                  [ 0.88314 ,  0.509881, -1.636363],
                  [ 0.      ,  1.391234,  1.158682],
                  [-1.204844, -0.695617,  1.158682],
                  [ 1.204844, -0.695617,  1.158682]]},
'HCOOH': {
    'description': "Formic Acid (HCOOH), HOCO cis, Cs symm.",
    'name': "HCOOH (formic acid)",
    'enthalpy': -90.5,
    'ZPE': 20.9525,
    'thermal correction': 2.5853,
    'symbols': 'OCOHH',
    'magmoms': None,
    'positions': [[-1.040945, -0.436432,  0.      ],
                  [ 0.      ,  0.423949,  0.      ],
                  [ 1.169372,  0.103741,  0.      ],
                  [-0.64957 , -1.335134,  0.      ],
                  [-0.377847,  1.452967,  0.      ]]},
'HCOOCH3': {
    'description': "Methyl formate (HCOOCH3), Cs symm.",
    'name': "HCOOCH_3 (methyl formate)",
    'enthalpy': -85.0,
    'ZPE': 38.3026,
    'thermal correction': 3.4726,
    'symbols': 'COOHCHHH',
    'magmoms': None,
    'positions': [[-0.931209, -0.083866,  0.      ],
                  [-0.711019, -1.278209,  0.      ],
                  [ 0.      ,  0.886841,  0.      ],
                  [-1.92836 ,  0.374598,  0.      ],
                  [ 1.356899,  0.397287,  0.      ],
                  [ 1.980134,  1.288164,  0.      ],
                  [ 1.541121, -0.206172,  0.889397],
                  [ 1.541121, -0.206172, -0.889397]]},
'CH3CONH2': {
    'description': "Acetamide (CH3CONH2), C1 symm.",
    'name': "CH_3CONH_2 (acetamide)",
    'enthalpy': -57.0,
    'ZPE': 45.2566,
    'thermal correction': 3.9313,
    'symbols': 'OCNCHHHHH',
    'magmoms': None,
    'positions': [[  4.24546000e-01,   1.32702400e+00,   8.03400000e-03],
                  [  7.71580000e-02,   1.49789000e-01,  -4.24900000e-03],
                  [  9.85518000e-01,  -8.78537000e-01,  -4.89100000e-02],
                  [ -1.37147500e+00,  -2.88665000e-01,  -1.44000000e-04],
                  [  7.07952000e-01,  -1.82424900e+00,   1.69942000e-01],
                  [ -1.99722900e+00,   5.84922000e-01,  -1.75477000e-01],
                  [ -1.56084200e+00,  -1.03927000e+00,  -7.71686000e-01],
                  [ -1.63211300e+00,  -7.23007000e-01,   9.69814000e-01],
                  [  1.95313300e+00,  -6.31574000e-01,   1.11866000e-01]]},
'CH2NHCH2': {
    'description': "Aziridine (cyclic CH2-NH-CH2 ring), C2v symm.",
    'name': "C_2H_4NH (aziridine)",
    'enthalpy': 30.2,
    'ZPE': 43.3728,
    'thermal correction': 2.6399,
    'symbols': 'CNCHHHHH',
    'magmoms': None,
    'positions': [[-0.03845 , -0.397326,  0.739421],
                  [-0.03845 ,  0.875189,  0.      ],
                  [-0.03845 , -0.397326, -0.739421],
                  [ 0.903052,  1.268239,  0.      ],
                  [-0.955661, -0.604926,  1.280047],
                  [-0.955661, -0.604926, -1.280047],
                  [ 0.869409, -0.708399,  1.249033],
                  [ 0.869409, -0.708399, -1.249033]]},
'NCCN': {
    'description': "Cyanogen (NCCN). D*h symm.",
    'name': "NCCN (cyanogen)",
    'enthalpy': 73.3,
    'ZPE': 10.2315,
    'thermal correction': 2.9336,
    'symbols': 'NCCN',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.875875],
                  [ 0.      ,  0.      ,  0.690573],
                  [ 0.      ,  0.      , -0.690573],
                  [ 0.      ,  0.      , -1.875875]]},
'C2H6NH': {
    'description': "Dimethylamine, (CH3)2NH, Cs symm.",
    'name': "(CH_3)_2NH (dimethylamine)",
    'enthalpy': -4.4,
    'ZPE': 57.0287,
    'thermal correction': 3.3760,
    'symbols': 'CNCHHHHHHH',
    'magmoms': None,
    'positions': [[-0.02753 , -0.224702,  1.20488 ],
                  [-0.02753 ,  0.59247 ,  0.      ],
                  [-0.02753 , -0.224702, -1.20488 ],
                  [ 0.791501, -0.962742,  1.248506],
                  [ 0.039598,  0.421182,  2.083405],
                  [-0.97222 , -0.772987,  1.26175 ],
                  [ 0.805303,  1.17822 ,  0.      ],
                  [ 0.791501, -0.962742, -1.248506],
                  [ 0.039598,  0.421182, -2.083405],
                  [-0.97222 , -0.772987, -1.26175 ]]},
'CH3CH2NH2': {
    'description': "Trans-Ethylamine (CH3-CH2-NH2), Cs symm.",
    'name': "CH_3CH_2NH_2 (trans ethylamine)",
    'enthalpy': -11.3,
    'ZPE': 57.2420,
    'thermal correction': 3.3678,
    'symbols': 'CCNHHHHHHH',
    'magmoms': None,
    'positions': [[ 1.210014, -0.353598,  0.      ],
                  [ 0.      ,  0.575951,  0.      ],
                  [-1.305351, -0.087478,  0.      ],
                  [ 2.14931 ,  0.208498,  0.      ],
                  [ 1.201796, -0.99776 ,  0.884909],
                  [ 1.201796, -0.99776 , -0.884909],
                  [ 0.034561,  1.230963, -0.876478],
                  [ 0.034561,  1.230963,  0.876478],
                  [-1.372326, -0.69834 ,  0.813132],
                  [-1.372326, -0.69834 , -0.813132]]},
'H2CCO': {
    'description': "Ketene (H2C=C=O), C2v symm.",
    'name': "CH_2CO (ketene)",
    'enthalpy': -11.4,
    'ZPE': 19.5984,
    'thermal correction': 2.8075,
    'symbols': 'CCHHO',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      , -1.21934 ],
                  [ 0.      ,  0.      ,  0.09892 ],
                  [ 0.      ,  0.938847, -1.753224],
                  [ 0.      , -0.938847, -1.753224],
                  [ 0.      ,  0.      ,  1.27862 ]]},
'CH2OCH2': {
    'description': "Oxirane (cyclic CH2-O-CH2 ring), C2v symm.",
    'name': "C_2H_4O (oxirane)",
    'enthalpy': -12.6,
    'ZPE': 35.4204,
    'thermal correction': 2.5816,
    'symbols': 'COCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.73158 , -0.375674],
                  [ 0.      ,  0.      ,  0.86095 ],
                  [ 0.      , -0.73158 , -0.375674],
                  [ 0.919568,  1.268821, -0.594878],
                  [-0.919568,  1.268821, -0.594878],
                  [-0.919568, -1.268821, -0.594878],
                  [ 0.919568, -1.268821, -0.594878]]},
'CH3CHO': {
    'description': "Acetaldehyde (CH3CHO), Cs symm.",
    'name': "CH_3CHO (acetaldehyde)",
    'enthalpy': -39.7,
    'ZPE': 34.2288,
    'thermal correction': 3.0428,
    'symbols': 'OCHCHHH',
    'magmoms': None,
    'positions': [[ 1.218055,  0.36124 ,  0.      ],
                  [ 0.      ,  0.464133,  0.      ],
                  [-0.477241,  1.465295,  0.      ],
                  [-0.948102, -0.700138,  0.      ],
                  [-0.385946, -1.634236,  0.      ],
                  [-1.596321, -0.652475,  0.880946],
                  [-1.596321, -0.652475, -0.880946]]},
'OCHCHO': {
    'description': "Glyoxal (O=CH-CH=O). Trans, C2h symm.",
    'name': "HCOCOH (glyoxal)",
    'enthalpy': -50.7,
    'ZPE': 22.8426,
    'thermal correction': 3.2518,
    'symbols': 'CCOHOH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.75643 ,  0.      ],
                  [ 0.      , -0.75643 ,  0.      ],
                  [ 1.04609 ,  1.389916,  0.      ],
                  [-0.99994 ,  1.228191,  0.      ],
                  [-1.04609 , -1.389916,  0.      ],
                  [ 0.99994 , -1.228191,  0.      ]]},
'CH3CH2OH': {
    'description': "Ethanol (trans, CH3CH2OH), Cs symm.",
    'name': "CH_3CH_2OH (ethanol)",
    'enthalpy': -56.2,
    'ZPE': 49.3072,
    'thermal correction': 3.3252,
    'symbols': 'CCOHHHHHH',
    'magmoms': None,
    'positions': [[ 1.168181, -0.400382,  0.      ],
                  [ 0.      ,  0.559462,  0.      ],
                  [-1.190083, -0.227669,  0.      ],
                  [-1.946623,  0.381525,  0.      ],
                  [ 0.042557,  1.207508,  0.886933],
                  [ 0.042557,  1.207508, -0.886933],
                  [ 2.115891,  0.1448  ,  0.      ],
                  [ 1.128599, -1.037234,  0.885881],
                  [ 1.128599, -1.037234, -0.885881]]},
'CH3OCH3': {
    'description': "DimethylEther (CH3-O-CH3), C2v symm.",
    'name': "CH_3OCH_3 (dimethylether)",
    'enthalpy': -44.0,
    'ZPE': 49.1911,
    'thermal correction': 3.3139,
    'symbols': 'COCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  1.165725, -0.19995 ],
                  [ 0.      ,  0.      ,  0.60011 ],
                  [ 0.      , -1.165725, -0.19995 ],
                  [ 0.      ,  2.017769,  0.480203],
                  [ 0.891784,  1.21432 , -0.840474],
                  [-0.891784,  1.21432 , -0.840474],
                  [ 0.      , -2.017769,  0.480203],
                  [-0.891784, -1.21432 , -0.840474],
                  [ 0.891784, -1.21432 , -0.840474]]},
'CH2SCH2': {
    'description': "Thiooxirane (cyclic CH2-S-CH2 ring), C2v symm.",
    'name': "C_2H_4S (thiirane)",
    'enthalpy': 19.6,
    'ZPE': 33.9483,
    'thermal correction': 2.7290,
    'symbols': 'CSCHHHH',
    'magmoms': None,
    'positions': [[ 0.      , -0.739719, -0.792334],
                  [ 0.      ,  0.      ,  0.863474],
                  [ 0.      ,  0.739719, -0.792334],
                  [-0.91394 , -1.250142, -1.076894],
                  [ 0.91394 , -1.250142, -1.076894],
                  [ 0.91394 ,  1.250142, -1.076894],
                  [-0.91394 ,  1.250142, -1.076894]]},
'C2H6SO': {
    'description': "Dimethylsulfoxide (CH3)2SO, Cs symm.",
    'name': "(CH_3)_2SO (dimethyl sulfoxide)",
    'enthalpy': -36.2,
    'ZPE': 48.8479,
    'thermal correction': 4.1905,
    'symbols': 'SOCCHHHHHH',
    'magmoms': None,
    'positions': [[  2.00000000e-06,   2.31838000e-01,  -4.38643000e-01],
                  [  2.00000000e-05,   1.50074200e+00,   3.79819000e-01],
                  [  1.33952800e+00,  -8.09022000e-01,   1.80717000e-01],
                  [ -1.33954800e+00,  -8.08992000e-01,   1.80718000e-01],
                  [  1.25583500e+00,  -8.96385000e-01,   1.26682500e+00],
                  [ -2.27940400e+00,  -3.13924000e-01,  -6.86740000e-02],
                  [  1.30440700e+00,  -1.79332700e+00,  -2.92589000e-01],
                  [  2.27939500e+00,  -3.13974000e-01,  -6.86740000e-02],
                  [ -1.30444700e+00,  -1.79329800e+00,  -2.92587000e-01],
                  [ -1.25585700e+00,  -8.96355000e-01,   1.26682600e+00]]},
'CH3CH2SH': {
    'description': "ThioEthanol (CH3-CH2-SH), Cs symm.",
    'name': "C_2H_5SH (ethanethiol)",
    'enthalpy': -11.1,
    'ZPE': 46.1583,
    'thermal correction': 3.5900,
    'symbols': 'CCSHHHHHH',
    'magmoms': None,
    'positions': [[ 1.514343,  0.679412,  0.      ],
                  [ 0.      ,  0.826412,  0.      ],
                  [-0.756068, -0.831284,  0.      ],
                  [-2.035346, -0.427738,  0.      ],
                  [-0.32497 ,  1.376482,  0.885793],
                  [-0.32497 ,  1.376482, -0.885793],
                  [ 1.986503,  1.665082,  0.      ],
                  [ 1.854904,  0.137645,  0.885494],
                  [ 1.854904,  0.137645, -0.885494]]},
'CH3SCH3': {
    'description': "Dimethyl ThioEther (CH3-S-CH3), C2v symm.",
    'name': "CH_3SCH_3 (dimethyl sulfide)",
    'enthalpy': -8.9,
    'ZPE': 46.6760,
    'thermal correction': 3.6929,
    'symbols': 'CSCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  1.366668, -0.513713],
                  [ 0.      ,  0.      ,  0.664273],
                  [ 0.      , -1.366668, -0.513713],
                  [ 0.      ,  2.296687,  0.057284],
                  [ 0.891644,  1.34568 , -1.144596],
                  [-0.891644,  1.34568 , -1.144596],
                  [ 0.      , -2.296687,  0.057284],
                  [-0.891644, -1.34568 , -1.144596],
                  [ 0.891644, -1.34568 , -1.144596]]},
'H2CCHF': {
    'description': "Vinyl fluoride (H2C=CHF), Cs symm.",
    'name': "CH_2=CHF (vinyl fluoride)",
    'enthalpy': -33.2,
    'ZPE': 27.2785,
    'thermal correction': 2.7039,
    'symbols': 'CCFHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.437714,  0.      ],
                  [ 1.191923, -0.145087,  0.      ],
                  [-1.148929, -0.278332,  0.      ],
                  [-0.186445,  1.505778,  0.      ],
                  [ 1.291348, -1.222833,  0.      ],
                  [ 2.083924,  0.466279,  0.      ]]},
'CH3CH2Cl': {
    'description': "Ethyl chloride (CH3-CH2-Cl), Cs symm.",
    'name': "C_2H_5Cl (ethyl chloride)",
    'enthalpy': -26.8,
    'ZPE': 41.0686,
    'thermal correction': 3.1488,
    'symbols': 'CCClHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.807636,  0.      ],
                  [ 1.505827,  0.647832,  0.      ],
                  [-0.823553, -0.77997 ,  0.      ],
                  [-0.344979,  1.341649,  0.885248],
                  [-0.344979,  1.341649, -0.885248],
                  [ 1.976903,  1.634877,  0.      ],
                  [ 1.839246,  0.10425 ,  0.885398],
                  [ 1.839246,  0.10425 , -0.885398]]},
'H2CCHCl': {
    'description': "Vinyl chloride, H2C=CHCl, Cs symm.",
    'name': "CH_2=CHCl (vinyl chloride)",
    'enthalpy': 8.9,
    'ZPE': 26.3554,
    'thermal correction': 2.8269,
    'symbols': 'CCClHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.756016,  0.      ],
                  [ 1.303223,  1.028507,  0.      ],
                  [-0.631555, -0.85498 ,  0.      ],
                  [-0.771098,  1.516963,  0.      ],
                  [ 2.056095,  0.249427,  0.      ],
                  [ 1.632096,  2.061125,  0.      ]]},
'H2CCHCN': {
    'description': "CyanoEthylene (H2C=CHCN), Cs symm.",
    'name': "CH_2=CHCN (acrylonitrile)",
    'enthalpy': 43.2,
    'ZPE': 31.4081,
    'thermal correction': 3.2034,
    'symbols': 'CCCHHHN',
    'magmoms': None,
    'positions': [[-0.161594, -1.638625,  0.      ],
                  [ 0.584957, -0.524961,  0.      ],
                  [ 0.      ,  0.782253,  0.      ],
                  [-1.245203, -1.598169,  0.      ],
                  [ 0.305973, -2.616405,  0.      ],
                  [ 1.669863, -0.572107,  0.      ],
                  [-0.467259,  1.867811,  0.      ]]},
'CH3COCH3': {
    'description': "Acetone (CH3-CO-CH3), C2v symm.",
    'name': "CH_3COCH_3 (acetone)",
    'enthalpy': -51.9,
    'ZPE': 51.5587,
    'thermal correction': 3.9878,
    'symbols': 'OCCCHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.405591],
                  [ 0.      ,  0.      ,  0.17906 ],
                  [ 0.      ,  1.28549 , -0.616342],
                  [ 0.      , -1.28549 , -0.616342],
                  [ 0.      ,  2.134917,  0.066535],
                  [ 0.      , -2.134917,  0.066535],
                  [-0.881086,  1.331548, -1.264013],
                  [ 0.881086,  1.331548, -1.264013],
                  [ 0.881086, -1.331548, -1.264013],
                  [-0.881086, -1.331548, -1.264013]]},
'CH3COOH': {
    'description': "Acetic Acid (CH3COOH), Single bonds trans, Cs symm.",
    'name': "CH_3COOH (acetic acid)",
    'enthalpy': -103.4,
    'ZPE': 38.1670,
    'thermal correction': 3.4770,
    'symbols': 'COOHCHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.15456 ,  0.      ],
                  [ 0.166384,  1.360084,  0.      ],
                  [-1.236449, -0.415036,  0.      ],
                  [-1.867646,  0.333582,  0.      ],
                  [ 1.073776, -0.892748,  0.      ],
                  [ 2.048189, -0.408135,  0.      ],
                  [ 0.968661, -1.528353,  0.881747],
                  [ 0.968661, -1.528353, -0.881747]]},
'CH3COF': {
    'description': "Acetyl fluoride (CH3COF), HCCO cis, Cs symm.",
    'name': "CH_3COF (acetyl fluoride)",
    'enthalpy': -105.7,
    'ZPE': 30.2742,
    'thermal correction': 3.3126,
    'symbols': 'COFCHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.186396,  0.      ],
                  [ 0.126651,  1.377199,  0.      ],
                  [-1.24395 , -0.382745,  0.      ],
                  [ 1.049454, -0.876224,  0.      ],
                  [ 2.035883, -0.417099,  0.      ],
                  [ 0.924869, -1.508407,  0.881549],
                  [ 0.924869, -1.508407, -0.881549]]},
'CH3COCl': {
    'description': "Acetyl,Chloride (CH3COCl), HCCO cis, Cs symm.",
    'name': "CH_3COCl (acetyl chloride)",
    'enthalpy': -58.0,
    'ZPE': 29.1855,
    'thermal correction': 3.5235,
    'symbols': 'CCClOHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.523878,  0.      ],
                  [ 1.486075,  0.716377,  0.      ],
                  [-0.452286, -1.217999,  0.      ],
                  [-0.845539,  1.37494 ,  0.      ],
                  [ 1.701027,  1.784793,  0.      ],
                  [ 1.917847,  0.240067,  0.882679],
                  [ 1.917847,  0.240067, -0.882679]]},
'C3H7Cl': {
    'description': "Propyl chloride (CH3CH2CH2Cl), Cs symm.",
    'name': "CH_3CH_2CH_2Cl (propyl chloride)",
    'enthalpy': -31.5,
    'ZPE': 58.6696,
    'thermal correction': 3.9885,
    'symbols': 'CCCHHHClHHHH',
    'magmoms': None,
    'positions': [[ 0.892629, -0.642344,  0.      ],
                  [ 2.365587, -0.245168,  0.      ],
                  [ 0.      ,  0.582921,  0.      ],
                  [ 0.663731, -1.252117,  0.879201],
                  [ 0.663731, -1.252117, -0.879201],
                  [ 3.005476, -1.130924,  0.      ],
                  [-1.73281 ,  0.139743,  0.      ],
                  [ 2.614882,  0.347704, -0.88473 ],
                  [ 2.614882,  0.347704,  0.88473 ],
                  [ 0.172881,  1.195836,  0.88646 ],
                  [ 0.172881,  1.195836, -0.88646 ]]},
'C2H6CHOH': {
    'description': "Isopropyl alcohol, (CH3)2CH-OH, Gauche isomer, C1 symm.",
    'name': "(CH_3)_2CHOH (isopropanol)",
    'enthalpy': -65.2,
    'ZPE': 66.5612,
    'thermal correction': 4.0732,
    'symbols': 'OCHHCCHHHHHH',
    'magmoms': None,
    'positions': [[  2.71910000e-02,   1.36369100e+00,  -1.67516000e-01],
                  [ -9.26000000e-04,   3.64590000e-02,   3.70128000e-01],
                  [  8.59465000e-01,   1.77564700e+00,   1.21307000e-01],
                  [  7.37100000e-03,   8.21450000e-02,   1.47050600e+00],
                  [ -1.31327500e+00,  -5.63514000e-01,  -8.89790000e-02],
                  [  1.20072100e+00,  -7.64480000e-01,  -1.04920000e-01],
                  [ -1.33400500e+00,  -6.07253000e-01,  -1.18100900e+00],
                  [  1.20284300e+00,  -8.07817000e-01,  -1.19718900e+00],
                  [ -2.14781200e+00,   5.49930000e-02,   2.47676000e-01],
                  [  2.13646200e+00,  -2.99324000e-01,   2.23164000e-01],
                  [ -1.43870900e+00,  -1.57427500e+00,   3.08340000e-01],
                  [  1.17773600e+00,  -1.78443600e+00,   2.89967000e-01]]},
'CH3CH2OCH3': {
    'description': "Methyl ethyl ether (CH3-CH2-O-CH3), Trans, Cs symm.",
    'name': "C_2H_5OCH_3 (methyl ethyl ether)",
    'enthalpy': -51.7,
    'ZPE': 66.6936,
    'thermal correction': 4.1058,
    'symbols': 'OCCCHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.006429, -0.712741,  0.      ],
                  [ 0.      ,  0.705845,  0.      ],
                  [ 1.324518, -1.226029,  0.      ],
                  [-1.442169,  1.160325,  0.      ],
                  [ 0.530962,  1.086484,  0.886881],
                  [ 0.530962,  1.086484, -0.886881],
                  [ 1.241648, -2.313325,  0.      ],
                  [ 1.881329, -0.905925, -0.89171 ],
                  [ 1.881329, -0.905925,  0.89171 ],
                  [-1.954863,  0.780605, -0.885855],
                  [-1.954863,  0.780605,  0.885855],
                  [-1.502025,  2.252083,  0.      ]]},
'C3H9N': {
    'description': "Trimethyl Amine, (CH3)3N, C3v symm.",
    'name': "(CH_3)_3N (trimethylamine)",
    'enthalpy': -5.7,
    'ZPE': 74.1584,
    'thermal correction': 4.0631,
    'symbols': 'NCCCHHHHHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.395846],
                  [ 0.      ,  1.378021, -0.065175],
                  [ 1.193401, -0.689011, -0.065175],
                  [-1.193401, -0.689011, -0.065175],
                  [ 0.      ,  1.461142, -1.167899],
                  [ 0.886156,  1.891052,  0.317655],
                  [-0.886156,  1.891052,  0.317655],
                  [ 1.265386, -0.730571, -1.167899],
                  [ 1.194621, -1.71296 ,  0.317655],
                  [ 2.080777, -0.178092,  0.317655],
                  [-1.265386, -0.730571, -1.167899],
                  [-2.080777, -0.178092,  0.317655],
                  [-1.194621, -1.71296 ,  0.317655]]},
'C4H4O': {
    'description': "Furan (cyclic C4H4O), C2v symm.",
    'name': "C_4H_4O (furan)",
    'enthalpy': -8.3,
    'ZPE': 43.2116,
    'thermal correction': 2.9480,
    'symbols': 'OCCCCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.163339],
                  [ 0.      ,  1.0947  ,  0.348039],
                  [ 0.      , -1.0947  ,  0.348039],
                  [ 0.      ,  0.7132  , -0.962161],
                  [ 0.      , -0.7132  , -0.962161],
                  [ 0.      ,  2.049359,  0.851113],
                  [ 0.      , -2.049359,  0.851113],
                  [ 0.      ,  1.370828, -1.819738],
                  [ 0.      , -1.370828, -1.819738]]},
'C4H4S': {
    'description': "Thiophene (cyclic C4H4S), C2v symm.",
    'name': "C_4H_4S (thiophene)",
    'enthalpy': 27.5,
    'ZPE': 41.2029,
    'thermal correction': 3.1702,
    'symbols': 'SCCCCHHHH',
    'magmoms': None,
    'positions': [[  0.00000000e+00,   0.00000000e+00,   1.18975300e+00],
                  [  0.00000000e+00,   1.23387600e+00,  -1.47400000e-03],
                  [  0.00000000e+00,  -1.23387600e+00,  -1.47400000e-03],
                  [  0.00000000e+00,   7.09173000e-01,  -1.27232200e+00],
                  [  0.00000000e+00,  -7.09173000e-01,  -1.27232200e+00],
                  [  0.00000000e+00,   2.27534300e+00,   2.91984000e-01],
                  [  0.00000000e+00,  -2.27534300e+00,   2.91984000e-01],
                  [  0.00000000e+00,   1.32193400e+00,  -2.16723100e+00],
                  [  0.00000000e+00,  -1.32193400e+00,  -2.16723100e+00]]},
'C4H4NH': {
    'description': "Pyrrole (Planar cyclic C4H4NH), C2v symm.",
    'name': "C_4H_5N (pyrrole)",
    'enthalpy': 25.9,
    'ZPE': 50.9688,
    'thermal correction': 3.1156,
    'symbols': 'HNCCCCHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  2.129296],
                  [ 0.      ,  0.      ,  1.118684],
                  [ 0.      ,  1.124516,  0.333565],
                  [ 0.      , -1.124516,  0.333565],
                  [ 0.      ,  0.708407, -0.983807],
                  [ 0.      , -0.708407, -0.983807],
                  [ 0.      ,  2.112872,  0.770496],
                  [ 0.      , -2.112872,  0.770496],
                  [ 0.      ,  1.357252, -1.849085],
                  [ 0.      , -1.357252, -1.849085]]},
'C5H5N': {
    'description': "Pyridine (cyclic C5H5N), C2v symm.",
    'name': "C_5H_5N (pyridine)",
    'enthalpy': 33.6,
    'ZPE': 54.8230,
    'thermal correction': 3.3007,
    'symbols': 'NCCCCCHHHHH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  1.424672],
                  [ 0.      ,  0.      , -1.386178],
                  [ 0.      ,  1.144277,  0.720306],
                  [ 0.      , -1.144277,  0.720306],
                  [ 0.      , -1.196404, -0.672917],
                  [ 0.      ,  1.196404, -0.672917],
                  [ 0.      ,  0.      , -2.473052],
                  [ 0.      ,  2.060723,  1.307477],
                  [ 0.      , -2.060723,  1.307477],
                  [ 0.      , -2.155293, -1.183103],
                  [ 0.      ,  2.155293, -1.183103]]},
'H2': {
    'description': "H2. D*h symm.",
    'name': "H_2",
    'enthalpy': 0.0,
    'ZPE': 6.2908,
    'thermal correction': 2.0739,
    'ionization energy': 15.43,
    'symbols': 'HH',
    'magmoms': None,
    'positions': [[ 0.      ,  0.      ,  0.368583],
                  [ 0.      ,  0.      , -0.368583]]},
'SH': {
    'description': "SH radical, C*v symm.",
    'name': "HS",
    'enthalpy': 34.2,
    'ZPE': 3.7625,
    'thermal correction': 2.0739,
    'symbols': 'SH',
    'magmoms': [ 1.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.079083],
                  [ 0.      ,  0.      , -1.26533 ]]},
'CCH': {
    'description': "CCH radical, C*v symm.",
    'name': "CCH",
    'enthalpy': 135.1,
    'ZPE': 7.8533,
    'thermal correction': 2.7830,
    'symbols': 'CCH',
    'magmoms': [ 0.,  1.,  0.],
    'positions': [[ 0.      ,  0.      , -0.462628],
                  [ 0.      ,  0.      ,  0.717162],
                  [ 0.      ,  0.      , -1.527198]]},
'C2H3': {
    'description': "C2H3 radical, Cs symm, 2-A'.",
    'name': "C_2H_3 (2A')",
    'enthalpy': 71.6,
    'ZPE': 22.5747,
    'thermal correction': 2.5483,
    'symbols': 'CCHHH',
    'magmoms': [ 0.,  1.,  0.,  0.,  0.],
    'positions': [[ 0.049798, -0.576272,  0.      ],
                  [ 0.049798,  0.710988,  0.      ],
                  [-0.87675 , -1.151844,  0.      ],
                  [ 0.969183, -1.154639,  0.      ],
                  [-0.690013,  1.498185,  0.      ]]},
'CH3CO': {
    'description': "CH3CO radical, HCCO cis, Cs symm, 2-A'.",
    'name': "CH_3CO (2A')",
    'enthalpy': -2.4,
    'ZPE': 26.6070,
    'thermal correction': 3.0842,
    'symbols': 'CCHHHO',
    'magmoms': [ 0.1,  0.6,  0. ,  0. ,  0. ,  0.3],
    'positions': [[-0.978291, -0.647814,  0.      ],
                  [ 0.      ,  0.506283,  0.      ],
                  [-0.455551, -1.607837,  0.      ],
                  [-1.617626, -0.563271,  0.881061],
                  [-1.617626, -0.563271, -0.881061],
                  [ 1.195069,  0.447945,  0.      ]]},
'H2COH': {
    'description': "H2COH radical, C1 symm.",
    'name': "H_2COH (2A)",
    'enthalpy': -4.1,
    'ZPE': 23.1294,
    'thermal correction': 2.6726,
    'symbols': 'COHHH',
    'magmoms': [ 0.7,  0.3,  0. ,  0. ,  0. ],
    'positions': [[ 0.687448,  0.029626, -0.082014],
                  [-0.672094, -0.125648,  0.030405],
                  [-1.09185 ,  0.740282, -0.095167],
                  [ 1.122783,  0.975263,  0.225993],
                  [ 1.221131, -0.888116,  0.118015]]},
'CH3O': {
    'description': "CH3O radical, Cs symm, 2-A'.",
    'name': "CH_3O CS (2A')",
    'enthalpy': 4.1,
    'ZPE': 22.4215,
    'thermal correction': 2.4969,
    'symbols': 'COHHH',
    'magmoms': [ 0.,  1.,  0.,  0.,  0.],
    'positions': [[-0.008618, -0.586475,  0.      ],
                  [-0.008618,  0.799541,  0.      ],
                  [ 1.055363, -0.868756,  0.      ],
                  [-0.467358, -1.004363,  0.903279],
                  [-0.467358, -1.004363, -0.903279]]},
'CH3CH2O': {
    'description': "CH3CH2O radical, Cs symm, 2-A''.",
    'name': "CH_3CH_2O (2A'')",
    'enthalpy': -3.7,
    'ZPE': 39.4440,
    'thermal correction': 3.0158,
    'symbols': 'CCOHHHHH',
    'magmoms': [ 0.,  0.,  1.,  0.,  0.,  0.,  0.,  0.],
    'positions': [[  1.00475700e+00,  -5.68263000e-01,   0.00000000e+00],
                  [  0.00000000e+00,   5.88691000e-01,   0.00000000e+00],
                  [ -1.26006200e+00,   7.29000000e-04,   0.00000000e+00],
                  [  1.46956000e-01,   1.20468100e+00,   8.96529000e-01],
                  [  1.46956000e-01,   1.20468100e+00,  -8.96529000e-01],
                  [  2.01936300e+00,  -1.64100000e-01,   0.00000000e+00],
                  [  8.69340000e-01,  -1.18683200e+00,   8.88071000e-01],
                  [  8.69340000e-01,  -1.18683200e+00,  -8.88071000e-01]]},
'CH3S': {
    'description': "CH3S radical, Cs symm, 2-A'.",
    'name': "CH_3S (2A')",
    'enthalpy': 29.8,
    'ZPE': 21.9415,
    'thermal correction': 2.6054,
    'symbols': 'CSHHH',
    'magmoms': [ 0.,  1.,  0.,  0.,  0.],
    'positions': [[-0.003856,  1.106222,  0.      ],
                  [-0.003856, -0.692579,  0.      ],
                  [ 1.043269,  1.427057,  0.      ],
                  [-0.479217,  1.508437,  0.895197],
                  [-0.479217,  1.508437, -0.895197]]},
'C2H5': {
    'description': "C2H5 radical, Staggered, Cs symm, 2-A'.",
    'name': "C_2H_5 (2A')",
    'enthalpy': 28.9,
    'ZPE': 36.5675,
    'thermal correction': 3.0942,
    'symbols': 'CCHHHHH',
    'magmoms': [ 0.,  1.,  0.,  0.,  0.,  0.,  0.],
    'positions': [[-0.014359, -0.694617,  0.      ],
                  [-0.014359,  0.794473,  0.      ],
                  [ 1.006101, -1.104042,  0.      ],
                  [-0.517037, -1.093613,  0.884839],
                  [-0.517037, -1.093613, -0.884839],
                  [ 0.100137,  1.346065,  0.923705],
                  [ 0.100137,  1.346065, -0.923705]]},
'C3H7': {
    'description': "(CH3)2CH radical, Cs symm, 2-A'.",
    'name': "(CH_3)_2CH (2A')",
    'enthalpy': 21.5,
    'ZPE': 54.2928,
    'thermal correction': 3.8435,
    'symbols': 'CCCHHHHHHH',
    'magmoms': [ 1.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
    'positions': [[ 0.014223,  0.54385 ,  0.      ],
                  [ 0.014223, -0.199742,  1.291572],
                  [ 0.014223, -0.199742, -1.291572],
                  [-0.32289 ,  1.575329,  0.      ],
                  [ 0.221417,  0.459174,  2.138477],
                  [ 0.221417,  0.459174, -2.138477],
                  [-0.955157, -0.684629,  1.484633],
                  [ 0.767181, -0.995308,  1.286239],
                  [ 0.767181, -0.995308, -1.286239],
                  [-0.955157, -0.684629, -1.484633]]},
'C3H9C': {
    'description': "t-Butyl radical, (CH3)3C, C3v symm.",
    'name': "(CH_3)_3C (t-butyl radical)",
    'enthalpy': 12.3,
    'ZPE': 71.7833,
    'thermal correction': 4.6662,
    'symbols': 'CCCCHHHHHHHHH',
    'magmoms': [1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
    'positions': [[ 0.      ,  0.      ,  0.191929],
                  [ 0.      ,  1.478187, -0.020866],
                  [ 1.280147, -0.739093, -0.020866],
                  [-1.280147, -0.739093, -0.020866],
                  [ 0.      ,  1.731496, -1.093792],
                  [-0.887043,  1.945769,  0.417565],
                  [ 0.887043,  1.945769,  0.417565],
                  [ 1.49952 , -0.865748, -1.093792],
                  [ 2.128607, -0.204683,  0.417565],
                  [ 1.241564, -1.741086,  0.417565],
                  [-1.49952 , -0.865748, -1.093792],
                  [-1.241564, -1.741086,  0.417565],
                  [-2.128607, -0.204683,  0.417565]]},
'NO2': {
    'description': "NO2 radical, C2v symm, 2-A1.",
    'name': "NO_2",
    'enthalpy': 7.9,
    'ZPE': 5.4631,
    'thermal correction': 2.4366,
    'symbols': 'NOO',
    'magmoms': [ 1.,  0.,  0.],
    'positions': [[ 0.      ,  0.      ,  0.332273],
                  [ 0.      ,  1.118122, -0.14537 ],
                  [ 0.      , -1.118122, -0.14537 ]]},
}


def get_ionization_energy(name, vertical=True):
    """Return the experimental ionization energy.

    If vertical is True, the vertical ionization energy is returned if
    available.
    """
    if name not in data:
        raise KeyError('System %s not in database.' % name)
    elif 'ionization energy' not in data[name]:
        raise KeyError('No data on ionization energy for system %s.' % name)
    else:
        if vertical and 'vertical ionization energy' in data[name]:
            return data[name]['vertical ionization energy']
        else:
            return data[name]['ionization energy']


def get_atomization_energy(name):
    """Determine extrapolated experimental atomization energy.

    The atomization energy is extrapolated from experimental heats of
    formation at room temperature, using calculated zero-point energies
    and thermal corrections.

    The atomization energy is returned in kcal/mol = 43.36 meV:

    >>> from ase.units import *; print kcal / mol
    0.0433641146392

    """
    assert name in extra or name in g2
    d = data[name]
    e = d['enthalpy']
    z = d['ZPE']
    dh = d['thermal correction']
    ae = -e + z + dh
    for a in string2symbols(d['symbols']):
        h = data[a]['enthalpy']
        dh = data[a]['thermal correction']
        ae += h - dh
    return ae


def molecule(name, **kwargs):
    """Create molecule."""
    if name in atoms:
        if 'magmoms' not in kwargs:
            kwargs['magmoms'] = [data[name]['magmom']]
        return  Atoms(name, **kwargs)
    if name not in extra and name not in g2:
        raise NotImplementedError('System %s not in database.' % name)
    d = data[name]
    if 'magmoms' not in kwargs:
        kwargs['magmoms'] = d['magmoms']
    return Atoms(d['symbols'], d['positions'], **kwargs)


def latex(name):
    """Convert name to LaTeX"""
    s = '$'
    last = False
    for i in name:
        if i.isalpha():
            if not last:
                s = s + r'\rm{'
                last = True
        elif last:
            s = s + '}'
            last = False
        s = s + i
    if i.isalpha():
        s = s + '}'
    s = s.replace(' ', r'\ ') + '$'
    return s


def rest(name):
    """Convert name to reStructuredText."""
    s = ''
    while name:
        c = name[0]
        if c == '_':
            s += r'\ :sub:`%s`\ ' % name[1]
            name = name[2:]
        elif c == '^':
            s += r'\ :sup:`%s`\ ' % name[1]
            name = name[2:]
        else:
            s += c
            name = name[1:]
    return s


if __name__ == '__main__':
    # Compare experimental values from the Kresse/VASP article with those from
    # the Argonne NL homepage    
    from gpaw.testing.atomization_data import atomization_vasp
    print 'Name      Kresse   ANL Diff'
    err = []
    for name in g1:
        anl = get_atomization_energy(name)
        kresse = atomization_vasp[name][0]
        err.append(kresse - anl)
        print '%-10s %5.1f %5.1f % 3.1f' % (name, kresse, anl, kresse - anl)

    import numpy as np
    mae = np.abs(err).mean()
    print 'MAE:', mae
